/*
 * BaseCodeGen
 *
 * Copyright (c) 1998-2002 Karl Moss. All Rights Reserved.
 *
 * Developed in conjunction with the book "Java Servlets Developer's
 * Guide" published by McGraw-Hill/Osborne Media Group
 *
 * You may study, use, modify, and distribute this software for any
 * purpose.
 *
 * This software is provided WITHOUT WARRANTY either expressed or
 * implied.
 *
 * @author  Karl Moss
 * @date Nov 26, 2001
 */

package com.omh.codeGen;

import java.lang.reflect.Method;
import java.io.*;

/**
 * This is the base class for code generation. It contains
 * methods to read and process a template file.
 */
public abstract class BaseCodeGen
        implements CodeGen
{
    // The pattern that represents a single indentation in the
    // template.
    protected static String indentPattern = "#";

    // The string that represents a single indentation (tab)
    protected static String indentString = "    ";

    // The pattern that denotes a special tag. The tag must start
    // and end with this pattern
    protected static String tagPattern = "%";

    // The interface class
    Class interfaceClass = null;

    // Interface name
    String interfaceName;

    // Object name
    String objectName;

    // Current method number
    int methodNum = 0;

    /**
     * Sets the interface to use
     *
     * @param name Interface name
     */
    public void setInterfaceName(String name)
    {
        interfaceName = name;
    }

    /**
    * Sets the object to use
    *
    * @param name Object name
    */
    public void setObjectName(String name)
    {
        objectName = name;
    }

    /**
     * Gets the name of the interface class that we will use
     * to reflect upon to get all of the methods to generate
     *
     * @return Full class name of the interface
     */
    public String getInterfaceName()
    {
        return interfaceName;
    }

    /**
     * Gets the name of the server object that we are generating
     * proxies for.
     *
     * @return Full class name of the server object
     */
    public String getObjectName()
    {
        return objectName;
    }

    /**
     * Gets the relative name of the template file to read
     *
     * @return Relative name of the template file
     */
    public abstract String getTemplateName();

    /**
     * Gets the super class name for the generated client
     *
     * @return Name of the super class
     */
    public abstract String getSuperclass();

    /**
     * Generates the source code for the given method
     *
     * @param m Method to generate
     * @param numIndent Number of indentations (tabs)
     * @param out Print writer
     */
    public abstract void codeMethod(Method m, int numIndent,
                                    PrintWriter out)
        throws IOException;

    /**
     * Given the class name of the base object return
     * the name of the client class to generate. The package name
     * is not included.
     *
     * @param objectName Name of the object
     * @return Name of the client class to generate
     */
    public abstract String getClientName(String objectName);

    /**
     * Given the class name of the base object return
     * the name of the server class to generate. The package name
     * is not included.
     *
     * @param objectName Name of the object
     * @return Name of the server class to generate
     */
    public abstract String getServerName(String objectName);

    /**
     * Given the class name of the base object return
     * the name of the target class to generate. The package name
     * is not included.
     *
     * @param objectName Name of the object
     * @return Name of the target class to generate
     */
    public abstract String getTargetName(String objectName);

    /**
     * Generates the source file.
     */
    public void generate()
        throws IOException
    {
        // Attempt to open the template file
        BufferedReader in = openTemplate();

        // The target output file
        PrintWriter outFile = null;

        // Create a new in-memory output stream that will hold
        // the contents of the generated file. We will not create
        // the output file until all processing has completed.
        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        PrintWriter out = new PrintWriter(baos);

        try
        {
            // Process the template file. Read each line until
            // the end of file
            String line;

            while (true)
            {
                // Read the next line
                line = in.readLine();

                // readLine returns null if EOF
                if (line == null)
                {
                    break;
                }

                // Strip off any indentation characters
                int numIndent = 0;
                while ((line.length() > 0) &&
                    line.startsWith(indentPattern))
                {
                    numIndent++;
                    line = line.substring(indentPattern.length());
                }

                // Process any embedded tags
                process(line, numIndent, out);
            }

            // Flush the output stream
            out.flush();

            // Processing is complete. Write the generated source
            // code.
            String fileName = stripPackage(getObjectName());
            fileName = getTargetName(fileName) + ".java";
            System.out.println("Writing " + fileName);
            FileOutputStream fos = new FileOutputStream(fileName);
            outFile = new PrintWriter(fos);

            // Turn our buffered output stream into an input stream
            ByteArrayInputStream bais =
                new ByteArrayInputStream(baos.toByteArray());
            InputStreamReader isr = new InputStreamReader(bais);
            BufferedReader br = new BufferedReader(isr);

            // Read the contents of our buffer and dump it to the
            // output file
            while (true)
            {
                // Read the next line
                line = br.readLine();

                // readLine returns null when EOF is reached
                if (line == null)
                {
                    break;
                }

                // Output the line
                outFile.println(line);
            }
        }
        finally
        {
            // Always close properly
            if (in != null)
            {
                in.close();
            }
            if (outFile != null)
            {
                outFile.close();
            }
        }
    }

    /**
     * Processes the given line. This involves scanning the line
     * for any embedded tags. If no tags exist the line will be
     * printed to the output stream.
     *
     * @param line Line from the template file
     * @param numIndent Number of indentations (tabs)
     * @param out Print writer
     */
    protected void process(String line, int numIndent, PrintWriter out)
        throws IOException
    {
        // Look for tags until all have been processed
        while (line != null)
        {
            // Search for the tag pattern
            int begPos = line.indexOf(tagPattern);

            // If no tag pattern exists, exit
            if (begPos < 0)
            {
                break;
            }

            // We have a starting tag pattern; look for an ending
            // tag pattern
            int endPos = line.indexOf(tagPattern, begPos + 1);

            // No ending tag pattern, exit
            if (endPos < 0)
            {
                break;
            }

            // Get the tag name
            String tag = line.substring(begPos + 1, endPos);

            // Process the tag
            line = processTag(line, tag, begPos, numIndent, out);
        }

        // If the line is not null it must be written to the
        // output stream
        if (line != null)
        {
            out.println(indent(numIndent) + line);
        }
    }

    /**
     * Process the tag for the given line. This method may be
     * overridden; just be sure to call super.processTag().
     *
     * @param line Line from the template file
     * @param tag Tag name
     * @param pos Starting position of the tag in the line
     * @param numIndent Number of indentations (tabs)
     * @param out Print writer
     * @return Line after tag replacement or null if the replacement
     * was written directly to the output stream
     */
    protected String processTag(String line, String tag, int pos,
                                int numIndent, PrintWriter out)
        throws IOException
    {
        // Replacement code for the tag
        String code = null;

        if (tag.equals("GENERATOR_NAME"))
        {
            code = getClass().getName();
        }
        else if (tag.equals("TIMESTAMP"))
        {
            code = new java.util.Date().toString();
        }
        else if (tag.equals("CLIENT_NAME"))
        {
            String objectName = getObjectName();

            // Strip off the package name
            objectName = stripPackage(objectName);

            // Get the name of the client
            code = getClientName(objectName);
        }
        else if (tag.equals("SERVER_NAME"))
        {
            String objectName = getObjectName();

            // Strip off the package name
            objectName = stripPackage(objectName);

            // Get the name of the server
            code = getServerName(objectName);
        }
        else if (tag.equals("PACKAGE_NAME"))
        {
            code  = getPackageName(getInterfaceName());
            if (code == null)
            {
                code = "";
            }
            else if (code.length() > 0)
            {
                code += ".";
            }
        }
        else if (tag.equals("PACKAGE_STATEMENT"))
        {
            // Assume that the code is going in the same package
            // as the interface
            String p = getPackageName(getInterfaceName());

            // No package. Do not output a line
            if (p.length() == 0)
            {
                line = null;
            }
            else
            {
                code = "package " + p + ";";
            }
        }
        else if (tag.equals("OBJECT_NAME"))
        {
            code = getObjectName();
        }
        else if (tag.equals("SUPER_CLASS"))
        {
            code = getSuperclass();
        }
        else if (tag.equals("INTERFACE_NAME"))
        {
            code = getInterfaceName();
        }
        else if (tag.equals("METHODS"))
        {

            // Process the interface methods
            processMethods(numIndent, out);

            // All code was written directly to the output stream
            line = null;
        }
        else
        {
            // Unknown tag
            System.out.println("WARNING: Unknown tag '" + tag + "'");
            code = "<UNKNOWN TAG " + tag + ">";
        }

        // If a code replacement was created, replace it in the
        // line
        if (code != null)
        {
            line = line.substring(0, pos) + code +
            line.substring(pos + tag.length() + 2);
        }
        return line;
    }

    /**
     * Process the METHOD tag. This involves reflecting upon
     * the interface and generating proxy code for each method
     *
     * @param numIndent Number of indentations (tabs)
     * @param out Print writer
     */
    protected void processMethods(int numIndent, PrintWriter out)
        throws IOException
    {
        // Get the interface class
        Class c = getInterfaceClass();

        // Get all of the methods for the interface
        java.lang.reflect.Method methods[] = c.getMethods();

        // Loop for each method in the interface
        for (int i = 0; i < methods.length; i++)
        {
            // Only generate code for public methods
            int modifiers = methods[i].getModifiers();
            if (!java.lang.reflect.Modifier.isPublic(modifiers))
            {
                continue;
            }

            // Generate the code for the method
            codeMethod(methods[i], numIndent, out);
        }
    }

    /**
     * Opens the template file
     */
    protected BufferedReader openTemplate()
        throws IOException
    {
        // Attempt to open the template
        InputStream in =
            ClassLoader.getSystemResourceAsStream(getTemplateName());

        if (in == null)
        {
        throw new IOException("Template " +
                getTemplateName() + " not found");
        }

        // Convert to a buffered reader
        InputStreamReader read = new InputStreamReader(in);

        return new BufferedReader(read);
    }

    /**
     * Returns the interface class
     */
    protected Class getInterfaceClass()
        throws IOException
    {
        Class c = interfaceClass;

        // Attempt to load the class if we have not done so already
        if (c == null)
        {
            try
            {
                // Try to load the class
                c = Class.forName(getInterfaceName());
            }
            catch (ClassNotFoundException ex)
            {
                throw new IOException("Interface " +
                        getInterfaceName() + " not found");
            }
        }
        return c;
    }

    /**
     * Given the number of indents (tabs), return a string
     * with the proper indentation characters
     *
     * @param n Number of indents
     * @return String containing indentation characters
     */
    protected String indent(int n)
    {
        String s = "";
        for (int i = 0; i < n; i++)
        {
            s += indentString;
        }
        return s;
    }

    /**
     * Given the full name of a class strip off the package name
     *
     * @param fullName Full name of a class
     * @param Class name without the package
     */
    protected String stripPackage(String fullName)
    {
        String s = fullName;

        // If there is a package, strip it off. We'll assume that
        // the last '.' is the end of the package name
        int pos = fullName.lastIndexOf(".");
        if (pos > 0)
        {
            s = fullName.substring(pos + 1);
        }
        return s;
    }

    /**
     * Given the full name of a class return the package name
     *
     * @param fullName Full name of a class
     * @param Package name or "" if none
     */
    protected String getPackageName(String fullName)
    {
        String s = "";

        // We'll assume that the last '.' is the end of the package
        // name
        int pos = fullName.lastIndexOf(".");
        if (pos > 0)
        {
            s = fullName.substring(0, pos);
        }
        return s;
    }

    /**
     * Given an object type this method will decode it.
     * Arrays of objects are encoded by the VM like
     * [Ljava.lang.String;
     *
     * @param type Object type
     * @return Decoded type
     */
    public static String decodeType(String type)
    {
        String array = "";

        // Look for arrays which will start with '['
        while (type.startsWith("["))
        {
            array += "[]";
            type = type.substring(1);
        }

        // If the object was an array decode the type.
        if (array.length() != 0)
        {
            switch(type.charAt(0))
            {

            case 'B':
                type = "byte";
                break;
            case 'C':
                type = "char";
                break;
            case 'D':
                type = "double";
                break;
            case 'F':
                type = "float";
                break;
            case 'I':
                type = "int";
                break;
            case 'J':
                type = "long";
                break;
            case 'L':

                // Object types begin with 'L' and end with ';'. Strip
                // both off
                type = type.substring(1, type.length() - 1);
                break;
            case 'S':
                type = "short";
                break;
            case 'Z':
                type = "boolean";
                break;
            default:
                type = type.substring(1);
                break;
            }

            // Add the array dimensions
            type += array;
        }

        return type;
    }
}

