/*
 * ServletGenClient
 *
 * Copyright (c) 1998-2002 Karl Moss. All Rights Reserved.
 *
 * Developed in conjunction with the book "Java Servlets Developer's
 * Guide" published by McGraw-Hill/Osborne Media Group
 *
 * You may study, use, modify, and distribute this software for any
 * purpose.
 *
 * This software is provided WITHOUT WARRANTY either expressed or
 * implied.
 *
 * @author  Karl Moss
 * @date Nov 26, 2001
 */

package com.omh.codeGen;

import java.lang.reflect.Method;
import java.io.PrintWriter;
import java.io.IOException;

/**
 * This class will generate the client proxy necessary for
 * making HTTP tunneled method calls.
 */
public class ServletGenClient
    extends BaseCodeGen
{
    // Current method number
    int methodNum = 0;

    /**
     * Gets the relative name of the template file to read
     *
     * @return Relative name of the template file
     */
    public String getTemplateName()
    {
        return "com/omh/codeGen/Client.template";
    }

    /**
    * Gets the super class name for the generated client
    *
    * @return Name of the super class
    */
    public String getSuperclass()
    {
        return "com.omh.tunnel.client.TunnelClient";;
    }

    /**
     * Given the class name of the base object return
     * the name of the client class to generate. The package name
     * is not included.
     *
     * @param objectName Name of the object
     * @return Name of the client class to generate
     */
    public String getClientName(String objectName)
    {
        return "Remote" + objectName + "Client";
    }

    /**
     * Given the class name of the base object return
     * the name of the server class to generate. The package name
     * is not included.
     *
     * @param objectName Name of the object
     * @return Name of the server class to generate
     */
    public String getServerName(String objectName)
    {
        return "Remote" + objectName + "Server";
    }

    /**
     * Given the class name of the base object return
     * the name of the target class to generate. The package name
     * is not included.
     *
     * @param objectName Name of the object
     * @return Name of the target class to generate
     */
    public String getTargetName(String objectName)
    {
        return getClientName(objectName);
    }

    /**
     * Generates the code for the given method
     *
     * @param m Method to generate
     * @param numIndent Number of indentations (tabs)
     * @param out Print writer
     */
    public void codeMethod(Method m, int numIndent, PrintWriter out)
        throws IOException
    {
        String line;
        String tab = indent(numIndent);
        boolean throwsClassNotFoundException = false;

        // Get the method return type
        Class ret = m.getReturnType();
        String retName = decodeType(ret.getName());

        // Validate the return type to ensure we can marshal it
        if (!ServletGenServer.validateType(ret))
        {
            throw new IOException("Invalid return data type " +
                    retName);
        }

        // Get the method parameters
        Class params[] = m.getParameterTypes();

        // Get the exceptions thrown by the method
        Class exceptions[] = m.getExceptionTypes();

        // Generate the method signature
        line = "public " + retName + " " + m.getName() + "(";

        // Loop for each parameter
        for (int i = 0; i < params.length; i++)
        {
            // Validate the parameter type to ensure we can marshal it
            if (!ServletGenServer.validateType(params[i]))
            {
                throw new IOException("Invalid parameter " +
                        "data type " + retName);
            }

            // Insert a comma if necessary
            if (i > 0)
            {
                line += ", ";
            }

            // Call the parameters p0, p1, etc.
            line += decodeType(params[i].getName()) + " p" + i;
        }

        // Add the ending paren
        line += ")";

        // Write out the method signature
        out.println(tab + line);

        // Take care of any exceptions thrown by the method
        if (exceptions.length > 0)
        {
            line = "throws ";
            for (int i = 0; i < exceptions.length; i++)
            {
                // Insert a comma if necessary
                if (i > 0)
                {
                    line += ", ";
                }
                line += exceptions[i].getName();
            }
            out.println(tab + indent(1) + line);
        }

        // Start the method body
        numIndent++;
        tab = indent(numIndent);
        out.println(tab + "{");
        numIndent++;
        tab = indent(numIndent);

        // Generate the default return value
        if (!retName.equals("void"))
        {
            line = retName + " retValue = ";

            // Determine the default value
            if (retName.equals("boolean"))
            {
                line += "false;";
            }
            else if (retName.equals("char") ||
                     retName.equals("byte") ||
                     retName.equals("short") ||
                     retName.equals("int") ||
                     retName.equals("long") ||
                     retName.equals("float") ||
                     retName.equals("double"))
            {
                line += "0;";
            }
            else
            {
                line += "null;";
            }
            out.println(tab + line);
        }

        out.println(tab + "try {");
        numIndent++;
        tab = indent(numIndent);

        out.println(tab + "// Create an internal buffer");
        out.println(tab + "ByteArrayOutputStream baos = " +
                "new ByteArrayOutputStream();");
        out.println("");
        out.println(tab + "// Create an object stream to write " +
                "the request");
        out.println(tab + "ObjectOutputStream out =");
        out.println(tab + indent(1) + "(ObjectOutputStream) "+
                "_createHeader(baos, " + methodNum + ");");

        // Write the parameters
        for (int i = 0; i < params.length; i++)
        {
            String param = "p" + i;
            String paramType = decodeType(params[i].getName());

            // Convert scalars to the proper object
            if (paramType.equals("boolean"))
            {
                param = "new Boolean(" + param + ")";
            }
            else if (paramType.equals("byte"))
            {
                param = "new Byte(" + param + ")";
            }
            else if (paramType.equals("char"))
            {
                param = "new Character(" + param + ")";
            }
            else if (paramType.equals("short"))
            {
                param = "new Short(" + param + ")";
            }
            else if (paramType.equals("int"))
            {
                param = "new Integer(" + param + ")";
            }
            else if (paramType.equals("long"))
            {
                param = "new Long(" + param + ")";
            }
            else if (paramType.equals("float"))
            {
                param = "new Float(" + param + ")";
            }
            else if (paramType.equals("double"))
            {
                param = "new Double(" + param + ")";
            }

            out.println(tab + "out.writeObject(" + param + ");");
        }

        // Invoke the method
        out.println("");
        out.println(tab + "// Invoke the method");
        out.println(tab + "ObjectInputStream in = ");
        out.println(tab + indent(1) + "(ObjectInputStream) " +
                "_invokeMethod(baos.toByteArray());");

        // Get the return value if necessary
        if (!retName.equals("void"))
        {
            out.println("");
            out.println(tab + "// Get the return value");
            out.println(tab + "Object retObject = in.readObject();");
            if (retName.equals("boolean"))
            {
                out.println(tab + "retValue = " +
                    "((Boolean) retObject).booleanValue();");
            }
            else if (retName.equals("byte"))
            {
                out.println(tab + "retValue = " +
                    "((Byte) retObject).byteValue();");
            }
            else if (retName.equals("char"))
            {
                out.println(tab + "retValue = " +
                    "((Character) retObject).charValue();");
            }
            else if (retName.equals("short"))
            {
                out.println(tab + "retValue = " +
                    "((Short) retObject).shortValue();");
            }
            else if (retName.equals("int"))
            {
                out.println(tab + "retValue = " +
                    "((Integer) retObject).intValue();");
            }
            else if (retName.equals("long"))
            {
                out.println(tab + "retValue = " +
                    "((Long) retObject).longValue();");
            }
            else if (retName.equals("float"))
            {
                out.println(tab + "retValue = " +
                    "((Float) retObject).floatValue();");
            }
            else if (retName.equals("double"))
            {
                out.println(tab + "retValue = " +
                    "((Double) retObject).doubleValue();");
            }
            else
            {
                out.println(tab + "retValue = (" +
                    retName + ") retObject;");
            }
            throwsClassNotFoundException = true;
        }

        // Wrap up
        out.println(tab + "out.close();");
        out.println(tab + "in.close();");

        // End the try block
        numIndent--;
        out.println(indent(numIndent) + "}");
        out.println(indent(numIndent) +
            "catch (java.io.IOException ex) {");
        out.println(indent(numIndent + 1) + "ex.printStackTrace();");
        out.println(indent(numIndent) + "}");

        if (throwsClassNotFoundException)
        {
            out.println(indent(numIndent) +
                "catch (ClassNotFoundException ex) {");
            out.println(indent(numIndent + 1) + "ex.printStackTrace();");
            out.println(indent(numIndent) + "}");
        }

        out.println(indent(numIndent) +
            "catch (TunnelException ex) {");
        out.println(indent(numIndent + 1) + "ex.printStackTrace();");
        out.println(indent(numIndent) + "}");

        // Write the return value
        if (!retName.equals("void"))
        {
            out.println(indent(numIndent) + "return retValue;");
        }

        // End the method body
        numIndent--;
        out.println(indent(numIndent) + "}");
        out.println("");

        // Increment the method number
        methodNum++;
    }
}
