/*
 * ServletGenLiteClient
 *
 * Copyright (c) 1998-2002 Karl Moss. All Rights Reserved.
 *
 * Developed in conjunction with the book "Java Servlets Developer's
 * Guide" published by McGraw-Hill/Osborne Media Group
 *
 * You may study, use, modify, and distribute this software for any
 * purpose.
 *
 * This software is provided WITHOUT WARRANTY either expressed or
 * implied.
 *
 * @author  Karl Moss
 * @date Nov 27, 2001
 */

package com.omh.codeGen;

import java.lang.reflect.Method;
import java.io.PrintWriter;
import java.io.IOException;

/**
 * This class will generate the client proxy necessary for
 * making 'lite' HTTP tunneled method calls. Lite method calls
 * use simple data input/output streams that can be used
 * with JDK 1.0.2
 */
public class ServletGenLiteClient
    extends BaseCodeGen
{
    // Current method number
    int methodNum = 0;

    /**
    * <p>Gets the relative name of the template file to read
    *
    * @return Relative name of the template file
    */
    public String getTemplateName()
    {
        return "com/omh/codeGen/LiteClient.template";
    }

    /**
     * Gets the super class name for the generated client
     *
     * @return Name of the super class
     */
    public String getSuperclass()
    {
        return "com.omh.tunnel.client.TunnelLiteClient";
    }

    /**
     * Given the class name of the base object return
     * the name of the client class to generate. The package name
     * is not included.
     *
     * @param objectName Name of the object
     * @return Name of the client class to generate
     */
    public String getClientName(String objectName)
    {
        return "Remote" + objectName + "LiteClient";
    }

    /**
     * Given the class name of the base object return
     * the name of the server class to generate. The package name
     * is not included.
     *
     * @param objectName Name of the object
     * @return Name of the server class to generate
     */
    public String getServerName(String objectName)
    {
        return "Remote" + objectName + "LiteServer";
    }

    /**
     * Given the class name of the base object return
     * the name of the target class to generate. The package name
     * is not included.
     *
     * @param objectName Name of the object
     * @return Name of the target class to generate
     */
    public String getTargetName(String objectName)
    {
        return getClientName(objectName);
    }

    /**
     * Generates the code for the given method
     *
     * @param m Method to generate
     * @param numIndent Number of indentations (tabs)
     * @param out Print writer
     */
    public void codeMethod(Method m, int numIndent, PrintWriter out)
        throws IOException
    {
        String line;
        String tab = indent(numIndent);

        // Get the method return type
        Class ret = m.getReturnType();
        String retName = decodeType(ret.getName());

        // Validate the return type to ensure we can marshal it
        if (!ServletGenLiteServer.validateType(ret))
        {
            throw new IOException("Invalid return data type " +
                    retName);
        }

        // Get the method parameters
        Class params[] = m.getParameterTypes();

        // Get the exceptions thrown by the method
        Class exceptions[] = m.getExceptionTypes();

        // Generate the method signature
        line = "public " + retName + " " +
                m.getName() + "(";

        // Loop for each parameter
        for (int i = 0; i < params.length; i++)
        {

            // Validate the parameter type to ensure we can marshal it
            if (!ServletGenLiteServer.validateType(params[i]))
            {
                throw new IOException("Invalid parameter " +
                        "data type " + retName);
            }

            // Insert a comma if necessary
            if (i > 0)
            {
                line += ", ";
            }

            // Call the parameters p0, p1, etc.
            line += decodeType(params[i].getName()) + " p" + i;
        }

        // Add the ending paren
        line += ")";

        // Write out the method signature
        out.println(tab + line);

        // Take care of any exceptions thrown by the method
        if (exceptions.length > 0)
        {
            line = "throws ";
            for (int i = 0; i < exceptions.length; i++)
            {
                // Insert a comma if necessary
                if (i > 0)
                {
                    line += ", ";
                }
                line += exceptions[i].getName();
            }
            out.println(tab + indent(1) + line);
        }

        // Start the method body
        numIndent++;
        tab = indent(numIndent);
        out.println(tab + "{");
        numIndent++;
        tab = indent(numIndent);

        // Generate the default return value
        if (!retName.equals("void"))
        {
            line = retName + " retValue = ";

            // Determine the default value
            if (retName.equals("boolean"))
            {
                line += "false;";
            }
            else if (retName.equals("char") ||
                     retName.equals("byte") ||
                     retName.equals("short") ||
                     retName.equals("int") ||
                     retName.equals("long") ||
                     retName.equals("float") ||
                     retName.equals("double"))
            {
                line += "0;";
            }
            else
            {
                line += "null;";
            }
            out.println(tab + line);
        }

        out.println(tab + "try {");
        numIndent++;
        tab = indent(numIndent);

        out.println(tab + "// Create an internal buffer");
        out.println(tab + "ByteArrayOutputStream baos = " +
                "new ByteArrayOutputStream();");
        out.println("");
        out.println(tab + "// Create a data stream to write " +
                "the request");
        out.println(tab + "DataOutputStream out =");
        out.println(tab + indent(1) + "(DataOutputStream) "+
                "_createHeader(baos, " + methodNum + ");");

        // Write the parameters
        for (int i = 0; i < params.length; i++)
        {
            String write = "bad_type";
            String param = "p" + i;
            String paramType = decodeType(params[i].getName());
            boolean checkParamForNull = false;

            // Convert scalars to the proper object
            if (paramType.equals("boolean"))
            {
                write = "writeBoolean";
            }
            else if (paramType.equals("byte"))
            {
                write = "writeByte";
            }
            else if (paramType.equals("char"))
            {
                write = "writeChar";
            }
            else if (paramType.equals("short"))
            {
                write = "writeShort";
            }
            else if (paramType.equals("int"))
            {
                write = "writeInt";
            }
            else if (paramType.equals("long"))
            {
                write = "writeLong";
            }
            else if (paramType.equals("float"))
            {
                write = "writeFloat";
            }
            else if (paramType.equals("double"))
            {
                write = "writeDouble";
            }
            else if (paramType.equals("java.lang.String"))
            {
                write = "writeUTF";
                checkParamForNull = true;
            }
            else if (paramType.equals("byte[]"))
            {
                write = "write";
                checkParamForNull = true;
            }
            int indents = 0;
            if (checkParamForNull)
            {
                // Write a flag that indicates whether the value is
                // null or not
                out.println(tab + "out.writeBoolean((" + param +
                        " == null));");
                out.println(tab + "if (" + param + " != null)");
                indents = 1;
            }

            if (paramType.equals("byte[]"))
            {
                // Write the length if necessary
                out.println(tab + indent(1) +
                        "out.writeInt(" + param + ".length);");
                out.println(tab + "if (" + param + " != null)");
            }

            out.println(tab + indent(indents) + "out." + write + "(" +
                    param + ");");
        }

        // Invoke the method
        out.println("");
        out.println(tab + "// Invoke the method");
        out.println(tab + "DataInputStream in = ");
        out.println(tab + indent(1) + "(DataInputStream) " +
        "_invokeMethod(baos.toByteArray());");

        // Get the return value if necessary
        if (!retName.equals("void"))
        {
            boolean checkReturnForNull = false;
            out.println("");
            out.println(tab + "// Get the return value");
            String reader = "bad_type";
            if (retName.equals("boolean"))
            {
                reader = "readBoolean";
            }
            else if (retName.equals("byte"))
            {
                reader = "readByte";
            }
            else if (retName.equals("char"))
            {
                reader = "readChar";
            }
            else if (retName.equals("short"))
            {
                reader = "readShort";
            }
            else if (retName.equals("int"))
            {
                reader = "readInt";
            }
            else if (retName.equals("long"))
            {
                reader = "readLong";
            }
            else if (retName.equals("float"))
            {
                reader = "readFloat";
            }
            else if (retName.equals("double"))
            {
                reader = "readDouble";
            }
            else if (retName.equals("java.lang.String"))
            {
                reader = "readUTF";
                checkReturnForNull = true;
            }
            else if (retName.equals("byte[]"))
            {
                reader = "read";
                checkReturnForNull = true;
            }

            int indents = 0;
            String inParam = "";
            String inReturn = "retValue = ";

            // Check for null values
            if (checkReturnForNull)
            {
                out.println(tab + "boolean isNull = in.readBoolean();");

                // Allocate the byte array if necessary
                if (retName.equals("byte[]"))
                {
                    out.println(tab + "if (!isNull)");
                    out.println(tab + indent(1) +
                            "retValue = new byte[in.readInt()];");
                    inParam = "retValue";
                    inReturn = "";
                }
                out.println(tab + "if (!isNull)");
                indents = 1;
            }

            out.println(tab + indent(indents) + inReturn + "in." +
                    reader + "(" + inParam + ");");
        }

        // Wrap up
        out.println(tab + "out.close();");
        out.println(tab + "in.close();");

        // End the try block
        numIndent--;
        out.println(indent(numIndent) + "}");
        out.println(indent(numIndent) +
                "catch (java.io.IOException ex) {");
        out.println(indent(numIndent + 1) + "ex.printStackTrace();");
        out.println(indent(numIndent) + "}");

        out.println(indent(numIndent) +
                "catch (TunnelException ex) {");
        out.println(indent(numIndent + 1) + "ex.printStackTrace();");
        out.println(indent(numIndent) + "}");

        // Write the return value
        if (!retName.equals("void"))
        {
            out.println(indent(numIndent) + "return retValue;");
        }

        // End the method body
        numIndent--;
        out.println(indent(numIndent) + "}");
        out.println("");

        // Increment the method number
        methodNum++;
    }
}
