/*
 * ServletGenLiteServer
 *
 * Copyright (c) 1998-2002 Karl Moss. All Rights Reserved.
 *
 * Developed in conjunction with the book "Java Servlets Developer's
 * Guide" published by McGraw-Hill/Osborne Media Group
 *
 * You may study, use, modify, and distribute this software for any
 * purpose.
 *
 * This software is provided WITHOUT WARRANTY either expressed or
 * implied.
 *
 * @author  Karl Moss
 * @date Nov 26, 2001
 */

package com.omh.codeGen;

import java.lang.reflect.Method;
import java.io.PrintWriter;
import java.io.IOException;

/**
 * This class will generate the server stub necessary for
 * making 'lite' HTTP tunneled method calls. Lite method calls
 * use simple data input/output streams that can be used
 * with JDK 1.0.2
 */
public class ServletGenLiteServer
    extends BaseCodeGen
{
    // Current method number
    int methodNum = 0;

    /**
     * Gets the relative name of the template file to read
     *
     * @return Relative name of the template file
     */
    public String getTemplateName()
    {
        return "com/omh/codeGen/LiteServer.template";
    }

    /**
     * Gets the super class name for the generated client
     *
     * @return Name of the super class
     */
    public String getSuperclass()
    {
        return "com.omh.tunnel.server.TunnelLiteServer";
    }

    /**
     * Given the class name of the base object return
     * the name of the client class to generate. The package name
     * is not included.
     *
     * @param objectName Name of the object
     * @return Name of the client class to generate
     */
    public String getClientName(String objectName)
    {
        return "Remote" + objectName + "LiteClient";
    }

    /**
     * Given the class name of the base object return
     * the name of the server class to generate. The package name
     * is not included.
     *
     * @param objectName Name of the object
     * @return Name of the server class to generate
     */
    public String getServerName(String objectName)
    {
        return "Remote" + objectName + "LiteServer";
    }

    /**
     * Given the class name of the base object return
     * the name of the target class to generate. The package name
     * is not included.
     *
     * @param objectName Name of the object
     * @return Name of the target class to generate
     */
    public String getTargetName(String objectName)
    {
        return getServerName(objectName);
    }

    /**
     * Generates the code for the given method
     *
     * @param m Method to generate
     * @param numIndent Number of indentations (tabs)
     * @param out Print writer
     */
    public void codeMethod(Method m, int numIndent, PrintWriter out)
        throws IOException
    {
        String line;
        String tab = indent(numIndent);

        // Create the case statement
        out.println(tab + "case " + methodNum + ": //" +
        m.getName());

        numIndent++;
        tab = indent(numIndent);

        // Get the method return type
        Class ret = m.getReturnType();
        String retName = decodeType(ret.getName());

        // Validate the return type to ensure we can marshal it
        if (!validateType(ret))
        {
            throw new IOException("Invalid return data type " +
                    retName);
        }

        // Get the method parameters
        Class params[] = m.getParameterTypes();

        String paramLine = "";

        // Loop for each parameter
        for (int i = 0; i < params.length; i++)
        {
            // Insert commas if necessary
            if (i > 0)
            {
                paramLine += ", ";
            }

            // Name the parameter
            String thisParam = "p" + methodNum + "_" + i;
            paramLine += thisParam;

            // Validate the parameter type to ensure we can marshal it
            if (!validateType(params[i]))
            {
                throw new IOException("Invalid parameter " +
                        "data type " + retName);
            }

            String paramType = decodeType(params[i].getName());
            if (paramType.equals("boolean"))
            {
                out.println(tab + "boolean " + thisParam + " =");
                out.println(tab + indent(1) +
                    "((DataInputStream) in).readBoolean();");
            }
            else if (paramType.equals("byte"))
            {
                out.println(tab + "byte " + thisParam + " =");
                out.println(tab + indent(1) +
                    "((DataInputStream) in).readByte();");
            }
            else if (paramType.equals("char"))
            {
                out.println(tab + "char " + thisParam + " =");
                out.println(tab + indent(1) +
                    "((DataInputStream) in).readChar();");
            }
            else if (paramType.equals("short"))
            {
                out.println(tab + "short " + thisParam + " =");
                out.println(tab + indent(1) +
                    "((DataInputStream) in).readShort();");
            }
            else if (paramType.equals("int"))
            {
                out.println(tab + "int " + thisParam + " =");
                out.println(tab + indent(1) +
                    "((DataInputStream) in).readInt();");
            }
            else if (paramType.equals("long"))
            {
                out.println(tab + "long " + thisParam + " =");
                out.println(tab + indent(1) +
                    "((DataInputStream) in).readLong();");
            }
            else if (paramType.equals("float"))
            {
                out.println(tab + "float " + thisParam + " =");
                out.println(tab + indent(1) +
                    "((DataInputStream) in).readFloat();");
            }
            else if (paramType.equals("double"))
            {
                out.println(tab + "double " + thisParam + " =");
                out.println(tab + indent(1) +
                    "((DataInputStream) in).readDouble();");
            }
            else if (paramType.equals("byte[]"))
            {
                out.println(tab + paramType + " " + thisParam + " = null;");
                out.println(tab +
                    "if (!((DataInputStream) in).readBoolean()) {");
                out.println(tab + indent(1) + thisParam +
                    " = new byte[((DataInputStream) in).readInt()];");
                out.println(tab + indent(1) +
                    "((DataInputStream) in).read(" +
                    thisParam + ");");
                out.println(tab + "}");
            }
            else
            {
                out.println(tab + paramType + " " + thisParam + " = null;");
                out.println(tab +
                    "if (!((DataInputStream) in).readBoolean())");
                out.println(tab + indent(1) + thisParam + " = " +
                    "((DataInputStream) in).readUTF();");
            }
        }

        // Invoke the method
        line = "";
        if (!retName.equals("void"))
        {
            line += retName + " r" + methodNum + " = ";
        }

        line += "o." + m.getName() + "(" + paramLine + ");";
        out.println(tab + line);

        // Write the return value if necessary
        if (!retName.equals("void"))
        {
            line = "((DataOutputStream) out).write";
            int indents = 0;

            // Write the data properly
            if (retName.equals("boolean"))
            {
                line += "Boolean";
            }
            else if (retName.equals("byte"))
            {
                line += "Byte";
            }
            else if (retName.equals("char"))
            {
                line += "Char";
            }
            else if (retName.equals("short"))
            {
                line += "Short";
            }
            else if (retName.equals("int"))
            {
                line += "Int";
            }
            else if (retName.equals("long"))
            {
                line += "Long";
            }
            else if (retName.equals("float"))
            {
                line += "Float";
            }
            else if (retName.equals("double"))
            {
                line += "Double";
            }
            else if (retName.equals("java.lang.String"))
            {
                line += "UTF";
                indents = 1;

                // Write the null flag
                out.println(tab +
                    "((DataOutputStream) out).writeBoolean((r" +
                    methodNum + " == null));");

                out.println(tab + "if (r" + methodNum + " != null)");
            }
            else if (retName.equals("byte[]"))
            {
                indents = 1;

                // Write the null flag
                out.println(tab +
                    "((DataOutputStream) out).writeBoolean((r" +
                    methodNum + " == null));");

                // Write the length indicator
                out.println(tab + "if (r" + methodNum + " != null)");
                out.println(tab + indent(1) +
                    "((DataOutputStream) out).writeInt(r" +
                    methodNum + ".length);");
                out.println(tab + "if (r" + methodNum + " != null)");
            }

            out.println(tab + indent(indents) + line +
                "(r" + methodNum + ");");
        }

        // End the case statement
        out.println(tab + "break;");

        // Increment the method number
        methodNum++;
    }

    /**
     * Validates the given type to ensure that we can
     * marshal it properly.
     *
     * @param c Class of the data to marshal
     * @return true if the type is valid
     */
    protected static boolean validateType(Class c)
        throws IOException
    {
        boolean rc = false;

        String type = decodeType(c.getName());

        // Validate. Only scalars and Strings are valid
        if (type.equals("void") ||
            type.equals("boolean") ||
            type.equals("char") ||
            type.equals("byte") ||
            type.equals("byte[]") ||
            type.equals("short") ||
            type.equals("int") ||
            type.equals("long") ||
            type.equals("float") ||
            type.equals("double") ||
            type.equals("java.lang.String"))
        {
            rc = true;
        }

        return rc;
    }
}
