/*
 * ServletGenServer
 *
 * Copyright (c) 1998-2002 Karl Moss. All Rights Reserved.
 *
 * Developed in conjunction with the book "Java Servlets Developer's
 * Guide" published by McGraw-Hill/Osborne Media Group
 *
 * You may study, use, modify, and distribute this software for any
 * purpose.
 *
 * This software is provided WITHOUT WARRANTY either expressed or
 * implied.
 *
 * @author  Karl Moss
 * @date Nov 26, 2001
 */

package com.omh.codeGen;

import java.lang.reflect.Method;
import java.io.PrintWriter;
import java.io.IOException;

/**
 * This class will generate the server stub necessary for
 * making HTTP tunneled method calls.
 */
public class ServletGenServer
    extends BaseCodeGen
{
    // Current method number
    int methodNum = 0;

    /**
     * Gets the relative name of the template file to read
     *
     * @return Relative name of the template file
     */
    public String getTemplateName()
    {
        return "com/omh/codeGen/Server.template";
    }

    /**
     * Gets the super class name for the generated client
     *
     * @return Name of the super class
     */
    public String getSuperclass()
    {
        return "com.omh.tunnel.server.TunnelServer";
    }

    /**
     * Given the class name of the base object return
     * the name of the client class to generate. The package name
     * is not included.
     *
     * @param objectName Name of the object
     * @return Name of the client class to generate
     */
    public String getClientName(String objectName)
    {
        return "Remote" + objectName + "Client";
    }

    /**
     * Given the class name of the base object return
     * the name of the server class to generate. The package name
     * is not included.
     *
     * @param objectName Name of the object
     * @return Name of the server class to generate
     */
    public String getServerName(String objectName)
    {
        return "Remote" + objectName + "Server";
    }

    /**
     * Given the class name of the base object return
     * the name of the target class to generate. The package name
     * is not included.
     *
     * @param objectName Name of the object
     * @return Name of the target class to generate
     */
    public String getTargetName(String objectName)
    {
        return getServerName(objectName);
    }

    /**
     * Generates the code for the given method
     *
     * @param m Method to generate
     * @param numIndent Number of indentations (tabs)
     * @param out Print writer
     */
    public void codeMethod(Method m, int numIndent, PrintWriter out)
        throws IOException
    {
        String line;
        String tab = indent(numIndent);

        // Create the case statement
        out.println(tab + "case " + methodNum + ": //" +
                m.getName());

        numIndent++;
        tab = indent(numIndent);

        // Get the method return type
        Class ret = m.getReturnType();
        String retName = decodeType(ret.getName());

        // Validate the return type to ensure we can marshal it
        if (!validateType(ret))
        {
            throw new IOException("Invalid return data type " +
                    retName);
        }

        // Get the method parameters
        Class params[] = m.getParameterTypes();

        String paramLine = "";

        // Loop for each parameter
        for (int i = 0; i < params.length; i++)
        {
            // Insert commas if necessary
            if (i > 0)
            {
                paramLine += ", ";
            }

            // Name the parameter
            String thisParam = "p" + methodNum + "_" + i;
            paramLine += thisParam;

            // Validate the parameter type to ensure we can marshal it
            if (!validateType(params[i]))
            {
                throw new IOException("Invalid parameter " +
                        "data type " + retName);
            }

            String paramType = decodeType(params[i].getName());
            if (paramType.equals("boolean"))
            {
                out.println(tab + "boolean " + thisParam + " =");
                out.println(tab + indent(1) +
                    "((Boolean) read(in)).booleanValue();");
            }
            else if (paramType.equals("byte"))
            {
                out.println(tab + "byte " + thisParam + " =");
                out.println(tab + indent(1) +
                    "((Byte) read(in)).byteValue();");
            }
            else if (paramType.equals("char"))
            {
                out.println(tab + "char " + thisParam + " =");
                out.println(tab + indent(1) +
                    "((Character) read(in)).charValue();");
            }
            else if (paramType.equals("short"))
            {
                out.println(tab + "short " + thisParam + " =");
                out.println(tab + indent(1) +
                    "((Short) read(in)).shortValue();");
            }
            else if (paramType.equals("int"))
            {
                out.println(tab + "int " + thisParam + " =");
                out.println(tab + indent(1) +
                    "((Integer) read(in)).intValue();");
            }
            else if (paramType.equals("long"))
            {
                out.println(tab + "long " + thisParam + " =");
                out.println(tab + indent(1) +
                    "((Long) read(in)).longValue();");
            }
            else if (paramType.equals("float"))
            {
                out.println(tab + "float " + thisParam + " =");
                out.println(tab + indent(1) +
                    "((Float) read(in)).floatValue();");
            }
            else if (paramType.equals("double"))
            {
                out.println(tab + "double " + thisParam + " =");
                out.println(tab + indent(1) +
                    "((Double) read(in)).doubleValue();");
            }
            else
            {
                out.println(tab + paramType + " " + thisParam + " =");
                out.println(tab + indent(1) + "(" + paramType +
                    ") read(in);");
            }
        }

        // Invoke the method
        line = "";
        if (!retName.equals("void"))
        {
            line += retName + " r" + methodNum + " = ";
        }

        line += "o." + m.getName() + "(" + paramLine + ");";
        out.println(tab + line);

        // Write the return value if necessary
        if (!retName.equals("void"))
        {
            line = "((ObjectOutputStream) out).writeObject";

            String retType = "r" + methodNum;

            // Convert the scalars to objects
            if (retName.equals("boolean"))
            {
                retType = "new Boolean(" + retType + ")";
            }
            else if (retName.equals("byte"))
            {
                retType = "new Byte(" + retType + ")";
            }
            else if (retName.equals("char"))
            {
                retType = "new Character(" + retType + ")";
            }
            else if (retName.equals("short"))
            {
                retType = "new Short(" + retType + ")";
            }
            else if (retName.equals("int"))
            {
                retType = "new Integer(" + retType + ")";
            }
            else if (retName.equals("long"))
            {
                retType = "new Long(" + retType + ")";
            }
            else if (retName.equals("float"))
            {
                retType = "new Float(" + retType + ")";
            }
            else if (retName.equals("double"))
            {
                retType = "new Double(" + retType + ")";
            }
            out.println(tab + line + "(" + retType + ");");
        }

        // End the case statement
        out.println(tab + "break;");

        // Increment the method number
        methodNum++;
    }

    /**
     * Validates the given type to ensure that we can
     * marshal it properly.
     *
     * @param c Class of the data to marshal
     * @return true if the type is valid
     */
    protected static boolean validateType(Class c)
        throws IOException
    {
        boolean rc = false;

        String type = decodeType(c.getName());

        // Validate. All scalars are valid
        if (type.equals("void") ||
            type.equals("boolean") ||
            type.equals("char") ||
            type.equals("byte") ||
            type.equals("short") ||
            type.equals("int") ||
            type.equals("long") ||
            type.equals("float") ||
            type.equals("double"))
        {
            rc = true;
        }
        else if (c instanceof java.io.Serializable)
        {
            // The object must be serializable
            rc = true;
        }
        return rc;
    }
}
