/*
 * ShowClass
 *
 * Copyright (c) 1998-2002 Karl Moss. All Rights Reserved.
 *
 * Developed in conjunction with the book "Java Servlets Developer's
 * Guide" published by McGraw-Hill/Osborne Media Group
 *
 * You may study, use, modify, and distribute this software for any
 * purpose.
 *
 * This software is provided WITHOUT WARRANTY either expressed or
 * implied.
 *
 * @author  Karl Moss
 * @date Nov 27, 2001
 */

package com.omh.codeGen;

import java.util.*;
import java.lang.reflect.Modifier;
import java.lang.reflect.Field;
import java.lang.reflect.Constructor;
import java.lang.reflect.Method;

/**
 * This application displays information about the given
 * class including all of the classes that it extends, the
 * interfaces that it implements, and all of the declared
 * methods
 */
public class ShowClass
{
    /**
     * Main application entry point. A single argument is
     * required that specifies the class to display
     */
    public static void main(String args[])
    {
        // Get the class name
        if (args.length != 1)
        {
            System.out.println("Class must be specified.");
            return;
        }

        // Display the interfaces implemented
        ShowClass show = new ShowClass();
        show.go(args[0]);
    }

    /**
     * Given a class name display the classes extended,
     * interfaces implemented, and declared methods
     *
     * @param className Name of the class to process
     */
    public void go(String className)
    {
        try
        {
            // Attempt to load the given class

            Class c = Class.forName(className);

            // Get the list of classes that it extends
            Collection extendList = getSuperClasses(c);

            // Get the list of interfaces that this class implements
            Collection interfaceList = getInterfaces(c);

            // Get the list of declared fields for this class
            Collection fields = getFields(c);

            // Get the list of declared constructors for this class
            Collection ctors = getConstructors(c);

            // Get the list of declared methods for this class
            Collection methods = getMethods(c);

            // Display the class information
            System.out.println("\n" +
                    getModifierString(c.getModifiers()) +
                    " " + c.getName());

            // Display the extend list
            print(extendList, "extends");

            // Display the implements list
            print(interfaceList, "implements");

            // Display the fields
            print(fields, "Fields");

            // Display the constructors
            print(ctors, "Constructors");

            // Display the methods
            print(methods, "Methods");
        }
        catch (ClassNotFoundException ex)
        {
            System.out.println("Class '" + className + "' not found.");
        }
        catch (Exception ex)
        {
            ex.printStackTrace();
        }
    }

    /**
     * Prints the given collection
     *
     * @param c The collection
     * @param desc The collection description
     */
    protected void print(Collection c, String desc)
    {
        if (c != null)
        {
            Iterator i = c.iterator();
            boolean first = true;
            while (i.hasNext())
            {
                if (first)
                {
                    System.out.println(" " + desc + ":");
                    first = false;
                }
                System.out.println("  " + i.next());
            }
        }
    }

    /**
     * Return a list of all of the super classes for the given
     * class
     *
     * @param c Class to check
     * @return List of super classes
     */
    public Collection getSuperClasses(Class c)
    {
        List list = new ArrayList();

        // Get the first super class
        c = c.getSuperclass();

        // Loop while a class exists
        while (c != null)
        {
            // Add the super class name to the list
            list.add(c.getName());

            // Get the next super class
            c = c.getSuperclass();
        }
        return list;
    }

    /**
     * Returns a list containing all of the interfaces names
     * implemented by the given class. This includes not only
     * the interfaces implemented by the class, but all interfaces
     * implemented by any super classes as well
     *
     * @param c Class to check
     * @return List of implemented interfaces
     */
    public Collection getInterfaces(Class c)
    {
        // Keep a list of all of the implemented interfaces
        Set list = new TreeSet();

        // Loop while a class exists
        while (c != null)
        {
            // Get the interfaces for this class
            getInterfaces(c, list);

            // Get the next super class
            c = c.getSuperclass();
        }

        return list;
    }

    /**
     * Get the interfaces implemented for the given
     * class. This routine will be called recursively
     *
     * @param c Class to check
     * @param set Collection containing the list of all of the
     * implemented interfaces. Do not allow duplicates.
     */
    public void getInterfaces(Class c, Set list)
    {
        // If the class given is an interface add it to the list
        if (c.isInterface())
        {
            list.add(c.getName());
        }

        // Get the interfaces implemented for the class
        Class interfaces[] = c.getInterfaces();

        // Loop for each interface
        for (int i = 0; i < interfaces.length; i++)
        {
            // Get the interfaces extended for this interface
            getInterfaces(interfaces[i], list);
        }
    }

    /**
     * Returns a string description of the given modifier
     * flags (private, protected, public, interface, etc.)
     *
     * @param modifier Encoded modifiers
     * @return String containing list of modifiers
     */
    public String getModifierString(int modifier)
    {
        // Use the reflection API to do the work for us
        return Modifier.toString(modifier);
    }

    /**
     * Given an object type this method will decode it.
     * Arrays of objects are encoded by the VM like
     * [Ljava.lang.String;
     *
     * @param type Object type
     * @return Decoded type
     */
    public String decodeType(String type)
    {
        String array = "";

        // Look for arrays which will start with '['
        while (type.startsWith("["))
        {
            array += "[]";
            type = type.substring(1);
        }

        // If the object was an array decode the type.
        if (array.length() != 0)
        {
            switch(type.charAt(0))
            {
            case 'B':
                type = "byte";
                break;
            case 'C':
                type = "char";
                break;
            case 'D':
                type = "double";
                break;
            case 'F':
                type = "float";
                break;
            case 'I':
                type = "int";
                break;
            case 'J':
                type = "long";
                break;
            case 'L':
                // Object types begin with 'L' and end with ';'. Strip
                // both off
                type = type.substring(1, type.length() - 1);
                break;
            case 'S':
                type = "short";
                break;
            case 'Z':
                type = "boolean";
                break;
            default:
                type = type.substring(1);
                break;
            }

            // Add the array dimensions
            type += array;
        }

        return type;
    }

    /**
     * Returns a sorted list of declared fields for the
     * given class
     *
     * @param c Class to check
     * @return List of declared fields
     */
    public Collection getFields(Class c)
    {
        Map map = new TreeMap();
        List list = new ArrayList();

        // Get the list of declared fields
        Field f[] = c.getDeclaredFields();

        // Loop for each field
        for (int i = 0; i < f.length; i++)
        {
            // Get the name, type, and modifiers
            String name = f[i].getName();
            String type = f[i].getType().getName();
            String modifiers = getModifierString(f[i].getModifiers());

            // Save in sorted map. Key is field name
            map.put(name, modifiers + " " + decodeType(type)
                    + " " + name);
        }

        // Move the map sorted by name into our list. We need
        // to return the modifier + type + name for display
        Iterator iterator = map.values().iterator();
        while (iterator.hasNext())
        {
            list.add(iterator.next());
        }
        return list;
    }

    /**
     * Returns the list of declared contructors for the class
     *
     * @param c Class to check
     * @return List of declared constructors
     */
    public Collection getConstructors(Class c)
    {
        Map map = new TreeMap();
        List list = new ArrayList();

        // Get the list of declared constructors
        Constructor ctors[] = c.getDeclaredConstructors();

        // Loop for each constructor
        for (int i = 0; i < ctors.length; i++)
        {
            // Get the name, modifiers, and parameter types
            String name = ctors[i].getName();
            String modifiers = getModifierString(ctors[i].getModifiers());
            String params =
            getParameterString(ctors[i].getParameterTypes());

            // Save in the map; the key is the parameter list
            // since it will be unique
            map.put(params, modifiers + " " +
                    name + "(" + params + ")");
        }

        // Move the map sorted by name into our list. We need
        // to return the modifier + name + params
        Iterator iterator = map.values().iterator();
        while (iterator.hasNext())
        {
            list.add(iterator.next());
        }
        return list;
    }

    /**
     * Returns the list of declared methods for the class
     *
     * @param c Class to check
     * @return List of declared methods
     */
    public Collection getMethods(Class c)
    {
        Map map = new TreeMap();
        List list = new ArrayList();

        // Get the list of declared methods
        Method methods[] = c.getDeclaredMethods();

        // Loop for each method
        for (int i = 0; i < methods.length; i++)
        {
            // Get the name, type, modifiers, and parameter types
            String name = methods[i].getName();
            String type = methods[i].getReturnType().getName();
            String modifiers =
            getModifierString(methods[i].getModifiers());
            String params =
                getParameterString(methods[i].getParameterTypes());

            // Save in the map; the key is the method name and
            // parameter list
            map.put(name + " " + params, modifiers + " " +
                decodeType(type) + " " + name + "(" + params + ")");
        }

        // Move the map sorted by name into our list. We need
        // to return the modifier + type + name + params
        Iterator iterator = map.values().iterator();
        while (iterator.hasNext())
        {
            list.add(iterator.next());
        }
        return list;
    }

    /**
     * Given an array of parameter type information, return a
     * parameter list string
     *
     * @param params Array of parameter type information
     * @return Parameter list string
     */
    public String getParameterString(Class params[])
    {
        String s = "";

        // Loop for each parameter
        for (int i = 0; i < params.length; i++)
        {
            // Add a separating comma if necessary
            if (i > 0)
            {
                s += ", ";
            }

            // Add the parameter type name
            s += decodeType(params[i].getName());
        }

        return s;
    }
}
