/*
 * ConnectionServlet
 *
 * Copyright (c) 1998-2002 Karl Moss. All Rights Reserved.
 *
 * Developed in conjunction with the book "Java Servlets Developer's
 * Guide" published by McGraw-Hill/Osborne Media Group
 *
 * You may study, use, modify, and distribute this software for any
 * purpose.
 *
 * This software is provided WITHOUT WARRANTY either expressed or
 * implied.
 *
 * @author  Karl Moss
 * @date Nov 24, 2001
 */

package com.omh.db;

import com.omh.jdbc.ConnectionPool;
import com.omh.jdbc.ConnectionObject;

import javax.servlet.*;
import javax.servlet.http.*;
import java.util.*;
import java.io.*;
import java.sql.*;
import java.text.SimpleDateFormat;

/**
 * This is a simple servlet that holds a global connection
 * pool. The Servlet context is used to store a named attribute
 * so that other servlets have access to the connection pool
 */

public class ConnectionServlet
        extends HttpServlet
{
    // Our connection pool.
    ConnectionPool connectionPool;

    // Context attribute key
    public static String CONNECTION_POOL_KEY =
                    "com.omh.db.ConnectionServlet";

    // Used to format dates
    SimpleDateFormat formatter =
            new SimpleDateFormat("yyyy.MM.dd hh:mm:ss.SSS");

    /**
     * Initialize the servlet. This is called once when the
     * servlet is loaded. It is guaranteed to complete before any
     * requests are made to the servlet
     *
     * @param cfg Servlet configuration information
     */
    public void init(ServletConfig cfg)
            throws ServletException
    {
        super.init(cfg);

        // Create our connection pool
        connectionPool = new ConnectionPool();

        // Initialize the connection pool. This will start all
        // of the connections as specified in the connection
        // pool configuration file
        try
        {
            connectionPool.initialize();
        }
        catch (Exception ex)
        {
            // Convert the exception
            ex.printStackTrace();
            throw new ServletException
                ("Unable to initialize connection pool", ex);
        }

        // Add the connection pool to the context so that other servlets
        // can find us
        ServletContext context = getServletContext();
        context.setAttribute(CONNECTION_POOL_KEY, connectionPool);
    }

    /**
     * Destroy the servlet. This is called once when the servlet
     * is unloaded.
     */
    public void destroy()
    {
        // Remove the attribute from the context
        getServletContext().removeAttribute(CONNECTION_POOL_KEY);

        // Tear down our connection pool if it was created
        if (connectionPool != null)
        {
            connectionPool.destroy();
        }
        super.destroy();
    }

    public void doGet(HttpServletRequest req,
                      HttpServletResponse resp)
            throws ServletException, IOException
    {
        // Set the content type of the response
        resp.setContentType("text/html");

        PrintWriter out = resp.getWriter();

        // Print the HTML header
        out.println("<html>");
        out.println("<head>");
        out.println("<title>Connection Pool Status</title>");
        out.println("</head>");
        out.println("<h2><center>");
        out.println("Connection Pool Status");
        out.println("</center></h2>");
        out.println("<br><center>");

        if (connectionPool == null)
        {
            out.println("No pool active");
        }
        else
        {
            out.println("<table>");
            out.println("<tr><th>Connection #</th>");
            out.println("<th>In Use</th>");
            out.println("<th>Use Count</th>");
            out.println("<th>Last Accessed</th></tr>");

            Enumeration enum =
                    connectionPool.getConnectionPoolObjects();

            int i = 0;
            while (enum.hasMoreElements())
            {
                ConnectionObject co =
                        (ConnectionObject) enum.nextElement();

                // Output the stats
                out.println("<tr><td>" + i + "</td>");
                out.println("<td>" + co.isInUse() + "</td>");
                out.println("<td>" + co.getUseCount() + "</td>");
                out.println("<td>" + format(co.getLastAccess()) +
                            "</td></tr>");
                i++;
            }
            out.println("</table>");
        }
        out.println("</center>");

        // Wrap up
        out.println("</html>");
        out.flush();
        out.close();
    }

    /**
     * Formats the last accessed time into a human-readable
     * string. This method is synchronized because
     * the SimpleDateFormat object is not thread-safe.
     */
    protected synchronized String format(long ms)
    {
        String s = formatter.format(new java.util.Date(ms));
        return s;
    }
}

