/*
 * EmployeeList
 *
 * Copyright (c) 1998-2002 Karl Moss. All Rights Reserved.
 *
 * Developed in conjunction with the book "Java Servlets Developer's
 * Guide" published by McGraw-Hill/Osborne Media Group
 *
 * You may study, use, modify, and distribute this software for any
 * purpose.
 *
 * This software is provided WITHOUT WARRANTY either expressed or
 * implied.
 *
 * @author  Karl Moss
 * @date Nov 23, 2001
 */

package com.omh.db;

import javax.servlet.*;
import javax.servlet.http.*;
import java.util.*;
import java.io.*;
import java.sql.*;

/**
 * This is a simple servlet that will use JDBC to gather all
 * of the employee information from a database and format it
 * into an HTML table.
 */

public class EmployeeList
        extends HttpServlet
{
    public void doGet(HttpServletRequest req,
                      HttpServletResponse resp)
        throws ServletException, IOException
    {
        // Set the content type of the response
        resp.setContentType("text/html");

        PrintWriter out = resp.getWriter();

        // Print the HTML header
        out.println("<html>");
        out.println("<head>");
        out.println("<title>Employee List</title>");
        out.println("</head>");
        out.println("<h2><center>");
        out.println("Employees for Nezzer's Chocolate Factory");
        out.println("</center></h2>");
        out.println("<br>");

        // Create any addition properties necessary for connecting
        // to the database, such as user and password
        Properties props = new Properties();
        props.put("user", "karlmoss");
        props.put("password", "servlets");

        query("sun.jdbc.odbc.JdbcOdbcDriver",
              "jdbc:odbc:MyAccessDataSource",
              props,
              "SELECT Empno, Name, Position FROM Employee",
              out);

        // Wrap up
        out.println("</html>");
        out.flush();
        out.close();
    }

    /**
     * Given the JDBC driver name, URL, and query string,
     * execute the query and format the results into an
     * HTML table
     *
     * @param driverName JDBC driver name
     * @param connectionURL JDBC connection URL
     * @param props Addition connection properties, such as user
     * and password
     * @param query SQL query to execute
     * @param out PrintWriter to use to output the query results
     * @return true if the query was successful
     */
    private boolean query(String driverName, String connectionURL,
                          Properties props, String query,
                          PrintWriter out)
    {
        boolean rc = true;

        // The JDBC Connection object
        Connection con = null;

        // The JDBC Statement object
        Statement stmt = null;

        // The JDBC ResultSet object
        ResultSet rs = null;

        // Keep stats for how long it takes to execute
        // the query
        long startMS = System.currentTimeMillis();

        // Keep the number of rows in the ResultSet
        int rowCount = 0;

        try
        {

            // Create an instance of the JDBC driver so that it has
            // a chance to register itself
            Class.forName(driverName).newInstance();

            // Create a new database connection.
            con = DriverManager.getConnection(connectionURL, props);

            // Create a statement object that we can execute queries
            // with
            stmt = con.createStatement();

            // Execute the query
            rs = stmt.executeQuery(query);

            // Format the results into an HTML table
            rowCount = formatTable(rs, out);

        }
        catch (Exception ex)
        {
            // Send the error back to the client
            out.println("Exception!");
            ex.printStackTrace(out);
            rc = false;
        }
        finally
        {
            try
            {
                // Always close properly
                if (rs != null)
                {
                    rs.close();
                }
                if (stmt != null)
                {
                    stmt.close();
                }
                if (con != null)
                {
                    con.close();
                }
            }
            catch (Exception ex)
            {
                // Ignore any errors here
            }
        }

        // If we queried the table successfully, output some
        // statistics
        if (rc)
        {
            long elapsed = System.currentTimeMillis() - startMS;
            out.println("<br><i>" + rowCount + " rows in " +
            elapsed + "ms</i>");
        }

        return rc;
    }

    /**
     * Given a JDBC ResultSet, format the results into
     * an HTML table
     *
     * @param rs JDBC ResultSet
     * @param out PrintWriter to use to output the table
     * @return The number of rows in the ResultSet
     */
    private int formatTable(ResultSet rs, PrintWriter out)
            throws Exception
    {
        int rowCount = 0;

        // Create the table
        out.println("<center><table border>");

        // Process the results. First dump out the column
        // headers as found in the ResultSetMetaData
        ResultSetMetaData rsmd = rs.getMetaData();

        int columnCount = rsmd.getColumnCount();

        // Start the table row
        StringBuffer sb = new StringBuffer("<tr>");

        for (int i = 0; i < columnCount; i++)
        {

            // Create each table header. Note that the column index
            // is 1-based
            sb.append("<th>");
            sb.append(rsmd.getColumnLabel(i + 1));
            sb.append("</th>");
        }

        // End the table row
        sb.append("</tr>");
        out.println(sb.toString());

        // Now walk through the entire ResultSet and get each
        // row
        while (rs.next())
        {
            rowCount++;

            // Start a table row
            sb = new StringBuffer("<tr>");

            // Dump out the values of each row
            for (int i = 0; i < columnCount; i++)
            {

                // Create the table data. Note that the column index
                // is 1-based
                sb.append("<td>");
                sb.append(rs.getString(i + 1));
                sb.append("</td>");
            }

            // End the table row
            sb.append("</tr>");
            out.println(sb.toString());
        }

        // End the table
        out.println("</table></center>");

        return rowCount;
    }
}

