/*
 * IndyList
 *
 * Copyright (c) 1998-2002 Karl Moss. All Rights Reserved.
 *
 * Developed in conjunction with the book "Java Servlets Developer's
 * Guide" published by McGraw-Hill/Osborne Media Group
 *
 * You may study, use, modify, and distribute this software for any
 * purpose.
 *
 * This software is provided WITHOUT WARRANTY either expressed or
 * implied.
 *
 * @author  Karl Moss
 * @date Nov 23, 2001
 */

package com.omh.db;

import javax.servlet.*;
import javax.servlet.http.*;
import java.util.*;
import java.io.*;
import java.sql.*;

/**
 * This is a simple servlet that will use JDBC to gather all
 * of the Indy 500 winner  information from a database and format it
 * into an HTML table. This servlet uses HttpSessions to keep
 * track of the position within the ResultSet so that the
 * table can be split into several different pages, each with
 * a 'Next n rows' link.
 */

public class IndyList
        extends HttpServlet
{
    public void doGet(HttpServletRequest req,
                      HttpServletResponse resp)
        throws ServletException, IOException
    {
        // Get the last year shown on the page that
        // called us. Remember that we are sorting
        // the years in descending order.
        String lastYear = req.getParameter("lastYear");
        if (lastYear == null)
        {
            lastYear = "9999";
        }

        // Get our URI
        String uri = req.getRequestURI();

        // Set the content type of the response
        resp.setContentType("text/html");

        PrintWriter out = resp.getWriter();

        // Print the HTML header
        out.println("<html>");
        out.println("<head>");
        out.println("<title>Indianapolis 500 Winners</title>");
        out.println("</head>");
        out.println("<h2><center>");
        out.println("Indianapolis 500 Winners");
        out.println("</center></h2>");
        out.println("<br>");

        // Create any addition properties necessary for connecting
        // to the database, such as user and password
        Properties props = new Properties();
        props.put("user", "karlmoss");
        props.put("password", "servlets");

        query("sun.jdbc.odbc.JdbcOdbcDriver",
              "jdbc:odbc:MyAccessDataSource",
              props,
              "SELECT * from IndyWinners where year < " +
                    lastYear + " order by year desc",
              out,
              uri);

        // Wrap up
        out.println("</html>");
        out.flush();
        out.close();
    }

    public void doPost(HttpServletRequest req,
                       HttpServletResponse resp)
            throws ServletException, IOException
    {
        // Same as get
        doGet(req, resp);
    }

    /**
     * Given the JDBC driver name, URL, and query string,
     * execute the query and format the results into an
     * HTML table
     *
     * @param driverName JDBC driver name
     * @param connectionURL JDBC connection URL
     * @param props Addition connection properties, such as user
     * and password
     * @param query SQL query to execute
     * @param out PrintWriter to use to output the query results
     * @param uri Request URI
     * @return true if the query was successful
     */
    private boolean query(String driverName, String connectionURL,
                         Properties props, String query,
                         PrintWriter out, String uri)
    {
        boolean rc = true;

        // The JDBC Connection object
        Connection con = null;

        // The JDBC Statement object
        Statement stmt = null;

        // The JDBC ResultSet object
        ResultSet rs = null;

        // Keep stats for how long it takes to execute
        // the query
        long startMS = System.currentTimeMillis();

        // Keep the number of rows in the ResultSet
        int rowCount = 0;

        try
        {

            // Create an instance of the JDBC driver so that it has
            // a chance to register itself
            Class.forName(driverName).newInstance();

            // Create a new database connection.
            con = DriverManager.getConnection(connectionURL, props);

            // Create a statement object that we can execute queries
            // with
            stmt = con.createStatement();

            // Execute the query
            rs = stmt.executeQuery(query);

            // Format the results into an HTML table
            rowCount = formatTable(rs, out, uri);
        }
        catch (Exception ex)
        {
            // Send the error back to the client
            out.println("Exception!");
            ex.printStackTrace(out);
            rc = false;
        }
        finally
        {
            try
            {
                // Always close properly
                if (rs != null)
                {
                    rs.close();
                }
                if (stmt != null)
                {
                    stmt.close();
                }
                if (con != null)
                {
                    con.close();
                }
            }
            catch (Exception ex)
            {
                // Ignore any errors here
            }
        }

        // If we queried the table successfully, output some
        // statistics
        if (rc)
        {
            long elapsed = System.currentTimeMillis() - startMS;
            out.println("<br><i>" + rowCount + " rows in " +
            elapsed + "ms</i>");
        }

        return rc;
    }

    /**
     * Given a JDBC ResultSet, format the results into
     * an HTML table
     *
     * @param rs JDBC ResultSet
     * @param out PrintWriter to use to output the table
     * @param uri Requesting URI
     * @return The number of rows in the ResultSet
     */
    private int formatTable(ResultSet rs, PrintWriter out,
                            String uri)
            throws Exception
    {
        int rowsPerPage = 10;
        int rowCount = 0;

        // Keep track of the last year found
        String lastYear = "";

        // This will be true if there is still more data in the
        // table
        boolean more = false;

        // Create the table
        out.println("<center><table border>");

        // Process the results. First dump out the column
        // headers as found in the ResultSetMetaData
        ResultSetMetaData rsmd = rs.getMetaData();

        int columnCount = rsmd.getColumnCount();

        // Start the table row
        StringBuffer sb = new StringBuffer("<tr>");

        for (int i = 0; i < columnCount; i++)
        {

        // Create each table header. Note that the column index
        // is 1-based
        sb.append("<th>");
        sb.append(rsmd.getColumnLabel(i + 1));
        sb.append("</th>");
        }

        // End the table row
        sb.append("</tr>");
        out.println(sb.toString());

        // Now walk through the entire ResultSet and get each
        // row
        while (rs.next())
        {
            rowCount++;

            // Start a table row
            sb = new StringBuffer("<tr>");

            // Dump out the values of each row
            for (int i = 0; i < columnCount; i++)
            {

                // Create the table data. Note that the column index
                // is 1-based
                String data = rs.getString(i + 1);
                sb.append("<td>");
                sb.append(data);
                sb.append("</td>");

                // If this is the year column, cache it
                if (i == 0)
                {
                    lastYear = data;
                }
            }

            // End the table row
            sb.append("</tr>");
            out.println(sb.toString());

            // If we are keeping track of the maximum number of
            // rows per page and we have exceeded that count
            // break out of the loop
            if ((rowsPerPage > 0) &&
                (rowCount >= rowsPerPage))
            {
                // Find out if there are any more rows after this one
                more = rs.next();
                break;
            }
        }

        // End the table
        out.println("</table></center>");

        if (more)
        {

            // Create a 'Next' button
            out.println("<form method=POST action=\"" +
                        uri + "\">");
            out.println("<center>");
            out.println("<input type=submit value=\"Next " +
                        rowsPerPage + " rows\">");
            out.println("</center>");
            // Page was filled. Put in the last year that we saw
            out.println("<input type=hidden name=lastYear value=" +
                        lastYear + ">");
            out.println("</form>");
        }

        return rowCount;
    }
}

