/*
 * TransformationFilter
 *
 * Copyright (c) 1998-2002 Karl Moss. All Rights Reserved.
 *
 * Developed in conjunction with the book "Java Servlets Developer's
 * Guide" published by McGraw-Hill/Osborne Media Group
 *
 * You may study, use, modify, and distribute this software for any
 * purpose.
 *
 * This software is provided WITHOUT WARRANTY either expressed or
 * implied.
 *
 * @author  Karl Moss
 * @date Oct 26, 2001
 */

package com.omh.filters;

import javax.servlet.*;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.util.*;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.Transformer;
import javax.xml.transform.stream.StreamSource;
import javax.xml.transform.stream.StreamResult;

public class TransformationFilter
        implements Filter
{
    protected FilterConfig config;
    protected String agentMappingsFile;
    protected Map agentMappings;
    protected TransformerFactory factory;

    public void init(FilterConfig config)
        throws ServletException
    {
        // Save the config object
        this.config = config;

        // Get the name of the user agent to xsl template mapping
        // file
        agentMappingsFile =
                config.getInitParameter("agentMappings");
        if (agentMappingsFile == null)
        {
            throw new ServletException(
                    "agentMappings parameter is required");
        }

        // Attempt to read and load the agent mapping file
        loadAgentMapping(agentMappingsFile);

        // Create a new transformer factory that we'll use to create
        // transformers
        factory = TransformerFactory.newInstance();
    }

    public void destroy()
    {
    }

    public void doFilter(ServletRequest request,
                         ServletResponse response,
                         FilterChain chain)
        throws ServletException, IOException
    {
        ServletResponse newResponse = response;

        // If this is an HttpRequest, wrap the response
        // with our CharArrayWriter so that we can get
        // the contents
        if (request instanceof HttpServletRequest)
        {
            newResponse = new CharResponseWrapper(
                    (HttpServletResponse) response);
        }

        // Invoke the next filter/servlet in the chain
        // using (perhaps) a wrapped response
        chain.doFilter(request, newResponse);

        if (newResponse instanceof CharResponseWrapper)
        {
            // Get the contents of the output. A String
            // is returned only if a Writer was used,
            // indicating text output. If getOutputStream
            // was used the output will go to the "real"
            // underlying output stream back to the client
            String text = newResponse.toString();
            if (text != null)
            {
                HttpServletRequest hreq =
                        (HttpServletRequest) request;
                HttpServletResponse hres =
                        (HttpServletResponse) response;

                // Get the user agent and determine what
                // (if any) xsl template should be used
                // to transform the document
                String userAgent = hreq.getHeader("user-agent");

                // Find the xsl template for the user agent
                String xslFile = getTemplate(userAgent);

                // Debugging
                System.out.println("User agent " + userAgent +
                                   " using XSL template: " + xslFile);

                if (xslFile != null)
                {
                    // Apply the template and send back to the
                    // client
                    applyTemplate(text, xslFile, hres);
                }
                else
                {
                    // Write the output to the real
                    // output stream that goes back to
                    // the client
                    response.getWriter().write(text);
                }
            }
        }
    }

    protected void loadAgentMapping(String name)
        throws ServletException
    {
        InputStream in = null;

        // Get the resource
        try
        {
            in = config.getServletContext().getResourceAsStream(name);
            if (in == null)
            {
                // The resource was not found. Raise an exception
                throw new ServletException(name + " not found");
            }

            // Load the mappings into a Properties object
            Properties p = new Properties();
            p.load(in);

            // Calculate the path
            String path = "/";
            int pos = name.lastIndexOf("/");
            if (pos > 0)
            {
                path = name.substring(0, pos + 1);
            }

            // This SortedMap will be sorted with the longest
            // string first
            agentMappings =
                    new TreeMap(new LongestFirstComparator());

            // Put the properties into a sorted map so that
            // the user agents are sorted
            Enumeration enum = p.keys();
            while (enum.hasMoreElements())
            {
                String key = (String) enum.nextElement();
                String value = (String) p.get(key);

                // Prepend the path if the resource doesn't start
                // with a slash
                if (!value.startsWith("/"))
                {
                    value = path + value;
                }
                agentMappings.put(key, value);
            }
        }
        catch (IOException ex)
        {
            throw new ServletException(ex.getMessage(), ex);
        }
        finally
        {
            // Make sure the input stream gets closed
            if (in != null)
            {
                try
                {
                    in.close();
                }
                catch (Exception ex)
                {
                    // Ignore exceptions on close
                }
            }
        }
    }

    protected String getTemplate(String userAgent)
    {
        String file = null;

        // Rip through and find the first matching mapping
        // (the mappings are sorted longest first)
        Iterator i = agentMappings.keySet().iterator();
        while (i.hasNext())
        {
            String key = (String) i.next();
            if (userAgent.startsWith(key))
            {
                file = (String) agentMappings.get(key);
                break;
            }
        }

        // Find the default if one was given
        if (file == null)
        {
            file = (String) agentMappings.get("*");
        }
        return file;
    }

    protected void applyTemplate(String content,
                                 String name,
                                 HttpServletResponse response)
        throws ServletException, IOException
    {
        InputStream in = null;

        try
        {
            in = config.getServletContext().getResourceAsStream(name);
            if (in == null)
            {
                // The resource was not found. Raise an exception
                throw new ServletException(name + " not found");
            }

            // Create a transformer to work with the given style
            // sheet
            StreamSource source = new StreamSource(in);
            Transformer transformer =
                    factory.newTransformer(source);

            // Transform it!
            StringReader reader = new StringReader(content);
            source = new StreamSource(reader);

            // Buffer the results. We can't set any headers
            // (such as the content type) if output has been
            // committed to the response stream.
            StringWriter writer = new StringWriter();
            StreamResult result = new StreamResult(writer);
            transformer.transform(source, result);

            // Set the content type to the media-type output
            // property if one was specified in the XSL template
            String mediaType = transformer.getOutputProperty("media-type");
            if (mediaType != null)
            {
                response.setContentType(mediaType);
            }

            // Now stream the results to the "real" output
            // writer
            PrintWriter out = response.getWriter();
            out.print(writer.toString());
            out.close();
        }
        catch (Exception ex)
        {
            throw new ServletException(ex.getMessage(), ex);
        }
        finally
        {
            // Make sure the input stream gets closed
            if (in != null)
            {
                try
                {
                    in.close();
                }
                catch (Exception ex)
                {
                    // Ignore exceptions on close
                }
            }
        }
    }
}
