/*
 * SimpleMailReader
 *
 * Copyright (c) 1998-2002 Karl Moss. All Rights Reserved.
 *
 * Developed in conjunction with the book "Java Servlets Developer's
 * Guide" published by McGraw-Hill/Osborne Media Group
 *
 * You may study, use, modify, and distribute this software for any
 * purpose.
 *
 * This software is provided WITHOUT WARRANTY either expressed or
 * implied.
 *
 * @author  Karl Moss
 * @date Dec 19, 2001
 */

package com.omh.mail;

import javax.servlet.*;
import javax.servlet.http.*;
import javax.mail.MessagingException;
import javax.mail.Folder;
import javax.mail.Message;
import javax.mail.Flags;
import javax.mail.Address;
import javax.mail.Multipart;
import javax.mail.Part;
import javax.mail.Transport;
import javax.mail.internet.ContentType;
import javax.mail.internet.InternetAddress;
import javax.mail.internet.MimeMessage;
import java.io.IOException;
import java.io.PrintWriter;
import java.io.InputStream;
import java.util.Date;

/**
 * This is a simple mail reader that uses JavaMail to get
 * messages from a store. The user is first presented with
 * a login form and then, after successful login, can
 * read, delete, or send mail.
 */
public class SimpleMailReader
        extends HttpServlet
{
    public static String MAIL_USER = "SimpleMailReader.user";

    public static String FORM_TITLE = "Simple Mail Reader";
    public static String FORM_TRANSPORT = "transport";
    public static String FORM_TRANSPORTHOST = "transportHost";
    public static String FORM_STORE = "store";
    public static String FORM_STOREHOST = "storeHost";
    public static String FORM_USER = "user";
    public static String FORM_PASSWORD = "password";
    public static String FORM_ACTION = "action";
    public static String FORM_MSG = "msg";
    public static String FORM_PART = "part";
    public static String FORM_FROM = "from";
    public static String FORM_TO = "to";
    public static String FORM_CC = "cc";
    public static String FORM_SUBJECT = "subject";

    public static String ACTION_LOGIN = "Login";
    public static String ACTION_LOGOFF = "Logoff";
    public static String ACTION_VIEW = "View";
    public static String ACTION_WRITE = "New Message";
    public static String ACTION_SEND = "Send";

    public void doGet(HttpServletRequest req,
                      HttpServletResponse resp)
        throws ServletException, IOException
    {
        // Get the session object for this client session.
        // The parameter indicates to create a session
        // object if one does not exist
        HttpSession session = req.getSession(true);

        // Get the mail user for this session
        SimpleMailUser mailUser =
            (SimpleMailUser) session.getAttribute(MAIL_USER);

        // Create a mail user if one does not exist
        if (mailUser == null)
        {
            mailUser = new SimpleMailUser();
            session.setAttribute(MAIL_USER, mailUser);
        }

        if (!mailUser.isLoggedIn())
        {
            createLoginForm(req, resp, mailUser);
        }
        else
        {
            // Get the action from the form
            String action = req.getParameter(FORM_ACTION);

            // Determine what action to take
            if (ACTION_VIEW.equals(action))
            {
                // Get the message number
                String s = req.getParameter(FORM_MSG);
                int msg = 0;
                try
                {
                    msg = Integer.parseInt(s);
                }
                catch (Exception ex)
                {
                    // Ignore any errors and assume 0
                }

                // Get the part number
                s = req.getParameter(FORM_PART);
                int part = -1;
                try
                {
                    part = Integer.parseInt(s);
                }
                catch (Exception ex)
                {
                    // Ignore any errors and assume -1 (no part)
                }

                // If no part number was given, display the message
                if (part == -1)
                {
                    showMessage(req, resp, mailUser, msg);
                }
                else
                {
                    showPart(req, resp, mailUser, msg, part);
                }
            }
            else
            {
                showInbox(req, resp, mailUser);
            }
        }
    }

    public void doPost(HttpServletRequest req,
                       HttpServletResponse resp)
        throws ServletException, IOException
    {
        // Get the session object for this client session.
        // The parameter indicates to create a session
        // object if one does not exist
        HttpSession session = req.getSession(true);

        // Get the mail user for this session
        SimpleMailUser mailUser =
            (SimpleMailUser) session.getAttribute(MAIL_USER);

        // There should be a session
        if (mailUser == null)
        {
            createErrorPage(req, resp, "Invalid session", null);
            return;
        }

        // Get the action from the form
        String action = req.getParameter(FORM_ACTION);

        // Evaluate the action
        if (ACTION_LOGIN.equals(action))
        {
            // Attempt to login
            login(req, resp, mailUser);
        }
        else if (ACTION_LOGOFF.equals(action))
        {
            // Close and show the login page again
            mailUser.close();
            createLoginForm(req, resp, mailUser);
        }
        else if (ACTION_WRITE.equals(action))
        {
            // Create the form for writing a new message
            createWriteForm(req, resp, mailUser);
        }
        else if (ACTION_SEND.equals(action))
        {
            // Send the composed message
            sendMessage(req, resp, mailUser);
        }
    }

    /**
     * Attempt to login to the Store host
     *
     * @param req The request from the client
     * @param resp The response from the servlet
     * @param mailUser The current SimpleMailUser
     */
    public void login(HttpServletRequest req,
                      HttpServletResponse resp,
                      SimpleMailUser mailUser)
        throws ServletException, IOException
    {
        String transport = req.getParameter(FORM_TRANSPORT);
        String transportHost = req.getParameter(FORM_TRANSPORTHOST);
        String store = req.getParameter(FORM_STORE);
        String storeHost = req.getParameter(FORM_STOREHOST);
        String user = req.getParameter(FORM_USER);
        String password = req.getParameter(FORM_PASSWORD);

        // Create a session with the store and transport protocols
        //mailUser.createSession(store, transport);

        // Attempt to logon
        try
        {
            mailUser.login(store, storeHost,
            transport, transportHost,
            user, password);
        }
        catch (MessagingException ex)
        {
            createErrorPage(req, resp, "Unable to login", ex);
            return;
        }

        showInbox(req, resp, mailUser);
    }

    /**
     * Shows the all of the messages in the current user's inbox
     *
     * @param req The request from the client
     * @param resp The response from the servlet
     * @param mailUser The current SimpleMailUser
     */
    public void showInbox(HttpServletRequest req,
                          HttpServletResponse resp,
                          SimpleMailUser mailUser)
        throws ServletException, IOException
    {
        // Set the content type of the response
        resp.setContentType("text/html");

        // Get the PrintWriter to write the response
        PrintWriter out = resp.getWriter();

        // Set the response header to force the browser to
        // load the HTML page from the server instead of
        // from a cache
        resp.setHeader("Expires", "Tues, 01 Jan 1980 00:00:00 GMT");

        // Get the URI of this request
        String uri = req.getRequestURI();

        // Print a standard header
        out.println("<html>");
        out.println("<head>");
        out.println("<title>" + FORM_TITLE + "</title>");
        out.println("<body><center><h2>");
        out.println("INBOX for " + mailUser.getUser() +
                    " using " + mailUser.getStoreProtocol() +
                    " on host " + mailUser.getStoreHost());
        out.println("</h2>");

        try
        {
            // Get the inbox
            Folder inbox = mailUser.getInbox();

            // Get the number of messages in the inbox
            int n = inbox.getMessageCount();

            // Get the messages from the inbox
            Message[] msgs = inbox.getMessages();

            out.println("Total number of messages: " + n + "<br>");

            out.println("<form action=\"" + uri + "\" METHOD=\"POST\">");
            out.println("<input type=submit name=" +
                        FORM_ACTION + " value=\"" +
                        ACTION_LOGOFF + "\">");
            out.println("<input type=submit name=" +
                        FORM_ACTION + " value=\"" +
                        ACTION_WRITE + "\">");
                        out.println("</form>");

            out.println("<table border>");
            out.println("<tr><th></th>");
            out.println("<th>From</th>");
            out.println("<th>Sent</th>");
            out.println("<th>Subject</th></tr>");

            // Loop through the inbox
            for (int i = 0; i < n; i++)
            {
                Message m = msgs[i];

                // Skip deleted messages
                if (m.isSet(Flags.Flag.DELETED))
                {
                    continue;
                }

                out.println("<tr>");

                // Give the user somewhere to click to view the
                // message
                out.println("<td><a href=\"" +
                            uri + "?" + FORM_ACTION + "=" +
                            ACTION_VIEW + "&" + FORM_MSG + "=" + i +
                            "\">" + ACTION_VIEW + "</a>");

                // Show the from address
                Address from[] = m.getFrom();
                Address addr = null;
                if ((from != null) && (from.length > 0))
                {
                    addr = from[0];
                }

                out.println("<td>" + getAddress(addr) + "</td>");

                // Show the sent date
                Date date = m.getSentDate();
                String s = "";
                if (date != null)
                {
                    s = "" + date;
                }
                out.println("<td>" + s + "</td>");

                // Show the subject
                s = m.getSubject();
                if (s == null)
                {
                    s = "";
                }
                out.println("<td>" + s + "</td>");

                out.println("</tr>");
            }

            out.println("</table>");
        }
        catch (MessagingException ex)
        {
            out.println("<br>");
            out.println("ERROR: " + ex.getMessage());
        }

        // Wrap up
        out.println("</center>");
        out.println("</body>");
        out.println("</html>");
        out.flush();
    }

    /**
     * Shows the specified message
     *
     * @param req The request from the client
     * @param resp The response from the servlet
     * @param mailUser The current SimpleMailUser
     * @param msg The message number to display
     */
    public void showMessage(HttpServletRequest req,
                            HttpServletResponse resp,
                            SimpleMailUser mailUser, int msg)
        throws ServletException, IOException
    {
        // Set the content type of the response
        resp.setContentType("text/html");

        // Get the PrintWriter to write the response
        PrintWriter out = resp.getWriter();

        // Set the response header to force the browser to
        // load the HTML page from the server instead of
        // from a cache
        resp.setHeader("Expires", "Tues, 01 Jan 1980 00:00:00 GMT");

        // Get the URI of this request
        String uri = req.getRequestURI();

        // Print a standard header
        out.println("<html>");
        out.println("<head>");
        out.println("<title>" + FORM_TITLE + "</title>");
        out.println("<body>");

        try
        {
            // Get the inbox
            Folder inbox = mailUser.getInbox();

            // Get the messages from the inbox
            Message[] msgs = inbox.getMessages();

            // Get the requested message
            Message m = msgs[msg];

            // Show the date
            out.println("Date: " + m.getSentDate() + "<br>");

            // Show the from addresses.
            Address a[] = m.getFrom();
            out.println("From: " + formatAddresses(a) + "<br>");

            // Show the to addresses
            a = m.getRecipients(Message.RecipientType.TO);
            out.println("To: " + formatAddresses(a) + "<br>");

            // Show the copy addresses
            a = m.getRecipients(Message.RecipientType.CC);
            out.println("Cc: " + formatAddresses(a) + "<br>");

            // Show the subject
            String s = m.getSubject();
            if (s == null)
            {
                s = "";
            }
            out.println("Subject: <b>" + s + "</b><br><hr>");

            // Display the message
            Object o = m.getContent();

            // Figure out what kind of message we have
            if (m.isMimeType("text/plain"))
            {
                // Plain text message
                out.println("<pre>" + o + "</pre>");
            }
            else if (m.isMimeType("multipart/*"))
            {
                // Multi-part message
                Multipart mp = (Multipart) o;

                // Loop through the parts
                for (int j = 0; j < mp.getCount(); j++)
                {
                    Part part = mp.getBodyPart(j);

                    // Get the content type of this part
                    String contentType = part.getContentType();
                    if (contentType == null)
                    {
                        out.println("Bad content type for part " + j);
                        continue;
                    }
                    ContentType ct = new ContentType(contentType);

                    if (j != 0)
                    {
                        out.println("<hr>");
                    }

                    // Plain text part
                    if (ct.match("text/plain"))
                    {
                        out.println("<pre>" + part.getContent() + "</pre>");
                    }
                    else
                    {
                        String desc = "Attachment ";
                        s = part.getFileName();
                        if (s != null)
                        {
                            desc += s;
                        }

                        // Generate a URL for this part
                        out.println("<td><a href=\"" +
                                    uri + "?" + FORM_ACTION + "=" +
                                    ACTION_VIEW + "&" + FORM_MSG + "=" + msg +
                                    "&" + FORM_PART + "=" + j +
                                    "\">" + desc + "</a>");
                    }
                }
            }
            else
            {
                // Unknown MIME type
                out.println(m.getContentType());
            }
        }
        catch (MessagingException ex)
        {
            out.println("<br>");
            out.println("ERROR: " + ex.getMessage());
        }

        // Wrap up
        out.println("</body>");
        out.println("</html>");
        out.flush();
    }

    /**
     * Shows the specified part of a message
     *
     * @param req The request from the client
     * @param resp The response from the servlet
     * @param mailUser The current SimpleMailUser
     * @param msg The message number to display
     * @param part The part number to display
     */
    public void showPart(HttpServletRequest req,
                         HttpServletResponse resp,
                         SimpleMailUser mailUser,
                         int msg, int part)
        throws ServletException, IOException
    {
        // Get the PrintWriter to write the response
        PrintWriter out = resp.getWriter();

        try
        {
            // Get the inbox
            Folder inbox = mailUser.getInbox();

            // Get the messages from the inbox
            Message[] msgs = inbox.getMessages();

            // Get the requested message
            Message m = msgs[msg];

            // Get the requested part
            Multipart mp = (Multipart) m.getContent();
            Part p = mp.getBodyPart(part);

            // Set the content type
            String contentType = p.getContentType();
            if (contentType == null)
            {
                out.println("Invalid message part at " + part);
            }
            else
            {
                ContentType type = new ContentType(contentType);

                // Set the content type for the response to the browser
                resp.setContentType(type.getBaseType());

                // Copy the contents of the part to the output
                // stream
                InputStream in = p.getInputStream();
                int b;
                while (true)
                {
                    b = in.read();
                    if (b == -1)
                    {
                        break;
                    }
                    out.write(b);
                }
            }
        }
        catch (MessagingException ex)
        {
            out.println("<br>");
            out.println("ERROR: " + ex.getMessage());
        }

        out.flush();
    }

    /**
     * Gets the display address
     * @param address The mail address
     * @return The display name
     */
    protected String getAddress(Address address)
    {
        String name = "";
        if (address != null)
        {
            name = "" + address;

            if (address instanceof InternetAddress)
            {
                // We can get the "real" name from the InternetAddress
                String personal =
                ((InternetAddress) address).getPersonal();
                if (personal != null)
                {
                    name = personal;
                }
            }
        }
        return name;
    }

    /**
     * Formats the array of addresses
     * @param addresses An array of Address objects
     * @return A formatted string of names
     */
    protected String formatAddresses(Address addresses[])
    {
        String s = "";

        if (addresses != null)
        {
            for (int i = 0; i < addresses.length; i++)
            {
                if (i > 0)
                {
                    s += ", ";
                }
                s += getAddress(addresses[i]);
            }
        }

        return s;
    }

    /**
     * Creates the login form for the mail system
     *
     * @param req The request from the client
     * @param resp The response from the servlet
     * @param mailUser The current SimpleMailUser
     */
    public void createLoginForm(HttpServletRequest req,
                                HttpServletResponse resp,
                                SimpleMailUser mailUser)
        throws ServletException, IOException
    {
        // Set the content type of the response
        resp.setContentType("text/html");

        // Get the PrintWriter to write the response
        PrintWriter out = resp.getWriter();

        // Set the response header to force the browser to
        // load the HTML page from the server instead of
        // from a cache
        resp.setHeader("Expires", "Tues, 01 Jan 1980 00:00:00 GMT");

        // Get the URI of this request
        String uri = req.getRequestURI();

        // Print a standard header
        out.println("<html>");
        out.println("<head>");
        out.println("<title>" + FORM_TITLE + "</title>");
        out.println("</head>");
        out.println("<body>");
        out.println("<center><h2>Welcome to the Simple Mail Reader");
        out.println("<br><br>Please Log In</h2>");
        out.println("<form ACTION=\"" + uri +
                    "\" METHOD=\"POST\" ENCTYPE=" +
                    "\"application/x-www-form-urlencoded\">");
        out.println("<table>");

        // Create the combo box for the list of transport protocols
        out.println("<tr><td width=\"30%\">Transport Protocol:</td>");
        out.println("<td width=\"70%\"><select name=" +
                    FORM_TRANSPORT + " size=1>");
        String s[] = mailUser.getTransportNames();
        for (int i = 0; i < s.length; i++)
        {
            out.println("<option value=\"" + s[i] + "\">" +
            s[i].toUpperCase());
        }
        out.println("</select></td></tr>");

        // Create the input field for the transport host
        //out.println("<tr><td>Transport Host:</td>");
        //out.println("<td><input type=\"text\" name=" +
        //FORM_TRANSPORTHOST + " size=40></td></tr>");

        // Create the combo box for the list of store protocols
        out.println("<tr><td width=\"30%\">Store Protocol:</td>");
        out.println("<td width=\"70%\"><select name=" +
                    FORM_STORE + " size=1>");
        s = mailUser.getStoreNames();
        for (int i = 0; i < s.length; i++)
        {
            out.println("<option value=\"" + s[i] + "\">" +
                        s[i].toUpperCase());
        }
        out.println("</select></td></tr>");

        // Create the input field for the store host
        out.println("<tr><td>Mail Host:</td>");
        out.println("<td><input type=\"text\" name=" +
                    FORM_STOREHOST + " size=40></td></tr>");

        // Create the input field for the user name
        out.println("<tr><td>User Name:</td>");
        out.println("<td><input type=\"text\" name=" +
                    FORM_USER + " size=40></td></tr>");

        // Create the input field for the password
        out.println("<tr><td>Password:</td>");
        out.println("<td><input type=\"password\" name=" +
                    FORM_PASSWORD + " size=40></td></tr>");

        out.println("</table><br>");

        // Create the submit and reset buttons
        out.println("<input type=\"submit\" value=\"" +
                    ACTION_LOGIN + "\" name=" + FORM_ACTION + ">");
        out.println("<input type=\"reset\" value=\"Reset\">");

        out.println("</form></center>");

        // Wrap up
        out.println("</body>");
        out.println("</html>");
        out.flush();
    }

    /**
     * Creates the form for writing a new mail message
     *
     * @param req The request from the client
     * @param resp The response from the servlet
     * @param mailUser The current SimpleMailUser
     */
    public void createWriteForm(HttpServletRequest req,
                                HttpServletResponse resp,
                                SimpleMailUser mailUser)
        throws ServletException, IOException
    {
        // Set the content type of the response
        resp.setContentType("text/html");

        // Get the PrintWriter to write the response
        PrintWriter out = resp.getWriter();

        // Create the HTML form
        out.println("<html>");
        out.println("<head>");
        out.println("<title>" + FORM_TITLE + "</title>");
        out.println("<center><h2>New Message</h2>");
        out.println("<br><form method=POST action=\"" +
                    req.getRequestURI() + "\">");

        // Keep the last from name the user entered
        String fromName = mailUser.getFromName();
        if (fromName == null)
        {
            fromName = "";
        }

        out.println("<table>");

        // Format the 'from' input field
        out.println("<tr><td>From:</td>");
        out.println("<td><input type=text name=" +
                    FORM_FROM + " size=40 value=\"" + fromName +
                    "\"></td></tr>");

        // Format the 'to' input field
        out.println("<tr><td>To:</td>");
        out.println("<td><input type=text name=" +
                    FORM_TO + " size=40></td></tr>");

        // Format the 'cc' input field
        out.println("<tr><td>Cc:</td>");
        out.println("<td><input type=text name=" +
                    FORM_CC + " size=40></td></tr>");

        // Format the subject input field
        out.println("<tr><td>Subject:</td>");
        out.println("<td><input type=text name=" +
                    FORM_SUBJECT + " size=40></td></tr>");

        // Format the mail message input field
        out.println("<tr><td>Message:</td>");
        out.println("<td><textarea name=" + FORM_MSG +
                    " cols=40 rows=6></textarea></td></tr>");
        out.println("</table><br>");

        out.println("<input type=submit name=" +
                    FORM_ACTION + " value=\"" +
                    ACTION_SEND + "\">");

        out.println("<input type=reset value=\"Reset\">");
        out.println("</form></center></body></html>");

        // Wrap up
        out.println("</body>");
        out.println("</html>");
        out.flush();
    }

    /**
     * Sends the composed message. Note that the 'To' and 'Cc'
     * fields can contain multiple addresses separated by
     * either a comma or space.
     *
     * @param req The request from the client
     * @param resp The response from the servlet
     * @param mailUser The current SimpleMailUser
     */
    public void sendMessage(HttpServletRequest req,
                            HttpServletResponse resp,
                            SimpleMailUser mailUser)
        throws ServletException, IOException
    {
        // Set the content type of the response
        resp.setContentType("text/html");

        // Get the PrintWriter to write the response
        PrintWriter out = resp.getWriter();

        // Create the HTML form
        out.println("<html>");
        out.println("<head>");
        out.println("<title>" + FORM_TITLE + "</title>");
        out.println("<body>");

        // Get the form input fields
        String from = req.getParameter(FORM_FROM);
        String to = req.getParameter(FORM_TO);
        String cc = req.getParameter(FORM_CC);
        String subject = req.getParameter(FORM_SUBJECT);
        String msg = req.getParameter(FORM_MSG);

        // Validate
        if ((from == null) || (from.trim().length() == 0))
        {
            out.println("'From' name must be given");
        }
        else if ((to == null) || (to.trim().length() == 0))
        {
            out.println("'To' address(es) must be given");
        }
        else
        {
            try
            {
                // Create a new message
                Message m = new MimeMessage(mailUser.getSession());

                InternetAddress[] iAddr = null;

                // Set the 'from' address
                m.setFrom(new InternetAddress(from));

                // Save the 'from' address in the mail user object
                mailUser.setFromName(from);

                // Set the 'to' address(es)
                iAddr = InternetAddress.parse(to, false);
                m.setRecipients(Message.RecipientType.TO, iAddr);

                // Set the 'cc' address(es) if given
                if ((cc != null) && (cc.trim().length() > 0))
                {
                    iAddr = InternetAddress.parse(cc, false);
                    m.setRecipients(Message.RecipientType.CC, iAddr);
                }

                // Set the subject
                m.setSubject(subject);

                // Set the message
                m.setText(msg);

                // Send the message on it's way!
                Transport.send(m);

                // Redirect to ourselves to re-display the inbox
                String me = req.getRequestURI();
                resp.sendRedirect(me);
            }
            catch (Exception ex)
            {
                out.println("Unable to send message: " + ex.getMessage());
            }
        }

        // Wrap up
        out.println("</body>");
        out.println("</html>");
        out.flush();
    }

    /**
     * Creates a standard error page
     *
     * @param req The request from the client
     * @param resp The response from the servlet
     * @param msg A message to return to the user
     * @param ex An optional exception
     */
    public void createErrorPage(HttpServletRequest req,
                                HttpServletResponse resp,
                                String msg, Exception ex)
        throws ServletException, IOException
    {
        // Set the content type of the response
        resp.setContentType("text/html");

        // Set the response header to force the browser to
        // load the HTML page from the server instead of
        // from a cache
        resp.setHeader("Expires", "Tues, 01 Jan 1980 00:00:00 GMT");

        // Get the PrintWriter to write the response
        PrintWriter out = resp.getWriter();

        // Print a standard header
        out.println("<html>");
        out.println("<head>");
        out.println("<title>" + FORM_TITLE + "</title>");
        out.println("</head>");
        out.println("<body>");
        out.println("<center><h2>" + msg + "</h2>");
        out.println("</center>");
        if (ex != null)
        {
            out.println("<br>Stack Trace:<br>");
            ex.printStackTrace(out);
        }

        // Wrap up
        out.println("</body>");
        out.println("</html>");
        out.flush();
    }
}

