/*
 * SimpleMailUser
 *
 * Copyright (c) 1998-2002 Karl Moss. All Rights Reserved.
 *
 * Developed in conjunction with the book "Java Servlets Developer's
 * Guide" published by McGraw-Hill/Osborne Media Group
 *
 * You may study, use, modify, and distribute this software for any
 * purpose.
 *
 * This software is provided WITHOUT WARRANTY either expressed or
 * implied.
 *
 * @author  Karl Moss
 * @date Dec 19, 2001
 */

package com.omh.mail;

import javax.servlet.*;
import javax.servlet.http.*;
import javax.mail.Session;
import javax.mail.Store;
import javax.mail.Folder;
import javax.mail.Provider;
import javax.mail.MessagingException;
import javax.mail.URLName;
import java.util.Properties;
import java.util.Vector;

/**
 * This class encapsulates a mail user and is used by the
 * SimpleMailReader servlet.
 */
public class SimpleMailUser
    implements HttpSessionBindingListener
{
    Session session;
    boolean loggedIn;
    Store store;
    Folder inbox;
    String storeProtocol;
    String storeHost;
    String transportProtocol;
    String transportHost;
    String user;
    String fromName;

    /**
     * Creates a new SimpleMailUser
     */
    public SimpleMailUser()
    {
        // Create a default session
        Properties props = System.getProperties();
        session = Session.getDefaultInstance(props, null);
    }

    /**
     * Get the current session
     * @return The current session
     */
    public Session getSession()
    {
        return session;
    }

    /**
     * Gets the current inbox
     * @return The inbox
     */
    public Folder getInbox()
    {
        return inbox;
    }

    /**
     * Returns true if the current user is logged in
     * @return true if the current user is logged in
     */
    public boolean isLoggedIn()
    {
        return loggedIn;
    }

    /**
     * Gets the store protocol
     * @return The store protocol, such as POP3
     */
    public String getStoreProtocol()
    {
        return storeProtocol;
    }

    /**
     * Gets the store host
     * @return The store host name
     */
    public String getStoreHost()
    {
        return storeHost;
    }

    /**
     * Gets the transport protocol
     * @return The transport protocol, such as SMTP
     */
    public String getTransportProtocol()
    {
        return transportProtocol;
    }

    /**
     * Gets the transport host
     * @return The transport host name
     */
    public String getTransportHost()
    {
        return transportHost;
    }

    /**
     * Gets the current user name
     * @return The current user
     */
    public String getUser()
    {
        return user;
    }

    /**
     * Sets the last from address entered by the user
     * @param name The from address name
     */
    public void setFromName(String name)
    {
        fromName = name;
    }

    /**
     * Gets the last from address entered by the user
     * @return The from address name
     */
    public String getFromName()
    {
        return fromName;
    }

    /**
     * Returns the names of the currently available transport
     * protocols
     * @return An array of transport names
     */
    public String[] getTransportNames()
    {
        return getProtocolNames(Provider.Type.TRANSPORT);
    }

    /**
     * Returns the names of the currently available store
     * protocols
     * @return An array of store names
     */
    public String[] getStoreNames()
    {
        return getProtocolNames(Provider.Type.STORE);
    }

    /**
     * Logs on to the specified Mail store
     * @param store The store protocol
     * @param storeHost The store host name
     * @param transport The transport protocol
     * @param transportHost The transport host name
     * @param user The user name
     * @param password The user password
     */
    public void login(String storeName, String storeHost,
                      String transport, String transportHost,
                      String user, String password)
            throws MessagingException
    {
        try
        {
            // Create a new session
            close();
            Properties props = System.getProperties();
            props.put("mail.host", storeHost);
            session = Session.getDefaultInstance(props, null);

            // Create a new URLName
            URLName url = new URLName(storeName, storeHost, -1,
                                      "INBOX", user, password);

            store = session.getStore(url);
            store.connect();
            inbox = store.getFolder("INBOX");
            inbox.open(Folder.READ_WRITE);

            // Set properties
            this.storeProtocol = storeName;
            this.storeHost = storeHost;
            this.transportProtocol = transport;
            this.transportHost = transportHost;
            this.user = user;
            loggedIn = true;
        }
        finally
        {
            // If there is some failure make sure that everything
            // is closed down properly
            if (!loggedIn)
            {
                if (store != null)
                {
                    store.close();
                    store = null;
                }
            }
        }
    }

    /**
     * Returns the name of the given type of protocol
     * @return An array of names
     */
    protected String[] getProtocolNames(Provider.Type type)
    {
        // Use a vector to temporarily store the provider objects
        Vector v = new Vector();

        // Get a list of all of the current providers
        Provider p[] = session.getProviders();

        if (p != null)
        {
            for (int i = 0; i < p.length; i++)
            {
                // Only get the types requested
                if (p[i].getType() == type)
                {
                    // Store it in our vector
                    v.addElement(p[i]);
                }
            }
        }

        // Now create an array of names from the providers that
        // we put in our vector
        String s[] = new String[v.size()];
        for (int i = 0; i < v.size(); i++)
        {
            Provider prov = (Provider) v.elementAt(i);
            s[i] = prov.getProtocol();
        }
        return s;
    }

    /**
     * Called when this object is bound into a session.
     * @param event The event
     */
    public void valueBound(HttpSessionBindingEvent event)
    {
        // Nothing to do when the value is bound
    }

    /**
     * Called when this object is unbound from a session
     * @param event The event
     */
    public void valueUnbound(HttpSessionBindingEvent event)
    {
        // Close the store if necessary
        close();
    }

    /**
     * Close all connections
     */
    public void close()
    {
        loggedIn = false;

        // Close the inbox and store if necessary
        try
        {
            if (inbox != null)
            {
                // Close without expunging deleted messages
                inbox.close(false);
            }
            if (store != null)
            {
                store.close();
            }
        }
        catch (MessagingException ex)
        {
        // Ignore errors on close
        }
        inbox = null;
        store = null;
    }
}

