/*
 * MicroTunnelClient
 *
 * Copyright (c) 1998-2002 Karl Moss. All Rights Reserved.
 * 
 * Developed in conjunction with the book "Java Servlets Developer's
 * Guide" published by McGraw-Hill/Osborne Media Group
 *
 * You may study, use, modify, and distribute this software for any
 * purpose.
 *
 * This software is provided WITHOUT WARRANTY either expressed or
 * implied.
 *
 * @author  Karl Moss
 * @date Dec 14, 2001
 */

package com.omh.micro.client;

import javax.microedition.io.HttpConnection;
import javax.microedition.io.Connector;
import java.io.*;

public class MicroTunnelClient
{
    // The connection url
    String _url;

    // The server object handle
    private int objectHandle;

    // The session cookie
    private String sessionCookie;

    // The user agent. This consists of the CLDC version
    // and the MIDP version
    private static String userAgent;

    public MicroTunnelClient()
    {
        if (userAgent == null)
        {
            userAgent = "Profile:" +
                    System.getProperty("microedition.profiles") +
                    " Configuration:" +
                    System.getProperty("microedition.configuration");
        }
    }

    /**
     * Sets the URL of the connection
     *
     * @param url Connection URL
     */
    public void _setURL(String url)
    {
        _url = url;
    }

    /**
     * Gets the URL of the connection
     *
     * @return The connection URL
     */
    public String _getURL()
    {
        return _url;
    }

    /**
     * Initializes the client. Also makes a server request
     * to initialize the server as well.
     */
    public void _initialize() throws TunnelException
    {
        try
        {
            // Create a new buffer that will hold our data
            ByteArrayOutputStream buffer = new ByteArrayOutputStream();

            // Create a method header. An ordinal value of -1 is
            // reserved for initializing the server
            _createHeader(buffer, -1);

            // Invoke the method. This will send the initialization
            // header to the server
            DataInputStream in = _invokeMethod(buffer.toByteArray());

            // Get the handle to the object that was just
            // created
            objectHandle = in.readInt();

            // Close properly
            in.close();
        }
        catch (IOException ex)
        {
            // Re-throw as a tunnel exception
            ex.printStackTrace();
            throw new TunnelException(ex.getMessage());
        }
    }

    /**
     * Starts a method by creating the method header.
     * The header consists of the method ordinal to invoke.
     *
     * @param buffer Buffer to hold the header data
     * @param ordinal Method ordinal to invoke on the server
     * @return Output stream to be used to send parameters
     */
    public DataOutputStream _createHeader(ByteArrayOutputStream buffer,
             int ordinal)
        throws TunnelException
    {
        try
        {
            // Get an output stream use to write data to the buffer
            DataOutputStream out = new DataOutputStream(buffer);

            // Write the method ordinal
            out.writeInt(ordinal);

            // If we are not initializing the object we need to send
            // the object handle along with the header
            if (ordinal != -1)
            {
                out.writeInt(objectHandle);
            }

            out.flush();
            return out;
        }
        catch (IOException ex)
        {
            // Re-throw as a tunnel exception
            ex.printStackTrace();
            throw new TunnelException(ex.getMessage());
        }
    }

    /**
     * Sends the given buffer that will cause a remote
     * method to be invoked.
     *
     * @param buffer Buffer containing data to send to the server
     * @return Input stream to be used to read the response from
     * the server
     */
    public DataInputStream _invokeMethod(byte buf[])
        throws TunnelException
    {
        DataInputStream in = null;

        try
        {
            // Get the server URL
            String url = _getURL();
            if (url == null)
            {
                throw new IOException("Server URL has not been set");
            }

            // Attempt to connect to the host
            HttpConnection con = (HttpConnection) Connector.open(url);

            // Set the request method
            con.setRequestMethod(HttpConnection.POST);

            // Set the user agent
            con.setRequestProperty("User-Agent", userAgent);

            // Set the session cookie if we have one
            if (sessionCookie != null)
            {
                con.setRequestProperty("cookie", sessionCookie);
            }

            // Set the content that we are sending
            con.setRequestProperty("Content-type",
                                   "application/octet-stream");

            // Set the length of the data buffer we are sending
            con.setRequestProperty("Content-length",
                                   "" + buf.length);

            // Get the output stream to the server and send our
            // data buffer
            DataOutputStream out =
                    new DataOutputStream(con.openOutputStream());
            out.write(buf);

            // Flush the output stream and close it
            out.flush();
            out.close();

            // Get the response status
            int status = con.getResponseCode();
            if (status != HttpConnection.HTTP_OK)
            {
                throw new TunnelException("Invalid response: " + status);
            }

            // Get the input stream we can use to read the response
            in = new DataInputStream(con.openInputStream());

            // The server will always respond with an int value
            // that will either be the method ordinal that was
            // invoked, or a -2 indicating an exception was thrown
            // from the server
            int ordinal = in.readInt();

            // Check for an exception on the server.
            if (ordinal == -2)
            {
                // Read the exception message and throw it
                String msg = in.readUTF();
                throw new TunnelException(msg);
            }

            // Get the session cookie if we don't have one yet
            if (sessionCookie == null)
            {
                String cookie = con.getHeaderField("set-cookie");
                if (cookie != null)
                {
                    sessionCookie = parseCookie(cookie);
                }
            }
        }
        catch (IOException ex)
        {
            // Re-throw as a tunnel exception
            ex.printStackTrace();
            throw new TunnelException(ex.getMessage());
        }

        // Return the input stream to be used to read the rest
        // of the response from the server
        return in;
    }

    /**
     * Gets the cookie name and value from a cookie field
     */
    public static String parseCookie(String raw)
    {
        String c = raw;
        if (raw != null)
        {
            // Find the first ';'
            int endPos = raw.indexOf(';');

            if (endPos >= 0)
            {
                c = raw.substring(0, endPos);
            }
        }
        return c;
    }
}
