/*
 * Scheduler
 *
 * Copyright (c) 1998-2002 Karl Moss. All Rights Reserved.
 *
 * Developed in conjunction with the book "Java Servlets Developer's
 * Guide" published by McGraw-Hill/Osborne Media Group
 *
 * You may study, use, modify, and distribute this software for any
 * purpose.
 *
 * This software is provided WITHOUT WARRANTY either expressed or
 * implied.
 *
 * @author  Karl Moss
 * @date Oct 31, 2001
 */

package com.omh.scheduler;

import java.util.SortedSet;
import java.util.TreeSet;

/**
 * Maintains a list of scheduled Runnable objects. Each Runnable
 * will be run at the appropriate time. Each Runnable will be
 * run only once; you must re-schedule a Runnable after it is run
 * if you wish it to run again.
 */
public class Scheduler
        extends Thread
{
    // The list of scheduled runnable objects
    SortedSet queue;

    public Scheduler()
    {
        // Create a new queue using the proper comparator
        // that will ensure the proper sorting of
        // scheduled runnables
        queue = new TreeSet(new ScheduledRunnableComparator());

        // Start the scheduler
        start();
    }

    /**
     * Schedules a runnable
     * @param runnable The Runnable to execute
     * @param msFromNow The number of milliseconds from now to
     * execute
     */
    public synchronized void schedule(Runnable r, long msFromNow)
    {
        long time = System.currentTimeMillis() + msFromNow;

        // Create a new holder object
        ScheduledRunnable sr =
                new ScheduledRunnable(r, time);

        // Add to the queue. The Comparator will ensure the
        // proper sorting
        queue.add(sr);

        // Notify ourselves that the queue has been modified
        notify();
    }

    public synchronized void run()
    {
        try
        {
            while (true)
            {
                if (queue.size() == 0)
                {
                    // Nothing in the queue. Wait for something
                    // to be added
                    wait();
                }
                else
                {
                    long now = System.currentTimeMillis();

                    // Get the next runnable
                    ScheduledRunnable r =
                            (ScheduledRunnable) queue.first();
                    if (r.getTime() > now)
                    {
                        // Not ready to run yet. Wait for it
                        wait(r.getTime() - now);
                    }
                    else
                    {
                        // The Runnable is ready to go!
                        Runnable runnable = r.getRunnable();
                        if (runnable != null)
                        {
                            Thread t = new Thread(runnable);
                            t.setDaemon(true);
                            t.start();

                            // Remove the runnable from the
                            // queue
                            queue.remove(r);
                        }
                    }
                }
            }
        }
        catch (InterruptedException ie)
        {
            // If we are interrupted, exit
        }
    }
}
