/*
 * CustomHttpLogin
 *
 * Copyright (c) 1998-2002 Karl Moss. All Rights Reserved.
 *
 * Developed in conjunction with the book "Java Servlets Developer's
 * Guide" published by McGraw-Hill/Osborne Media Group
 *
 * You may study, use, modify, and distribute this software for any
 * purpose.
 *
 * This software is provided WITHOUT WARRANTY either expressed or
 * implied.
 *
 * @author  Karl Moss
 * @date Oct 22, 2001
 */

package com.omh.security;

import javax.servlet.*;
import javax.servlet.http.*;
import java.io.IOException;
import java.io.PrintWriter;

/**
 * This servlet uses HTTP Authentication to prompt for
 * a username and password and a custom authentication
 * routine to validate the user
 */
public class CustomHttpLogin
        extends HttpServlet
{
    // Define our counter key into the session
    static final String USER_KEY = "CustomLogin.user";

    public void doGet(HttpServletRequest req,
                      HttpServletResponse resp)
        throws ServletException, java.io.IOException
    {
        // Set the content type of the response
        resp.setContentType("text/html");

        // Get the PrintWriter to write the response
        PrintWriter out = resp.getWriter();

        // Set the response header to force the browser to
        // load the HTML page from the server instead of
        // from a cache
        resp.setHeader("Expires",
                "Tues, 01 Jan 1980 00:00:00 GMT");

        // Get the user for the current session
        HttpSession session = req.getSession();
        String sessionUser = null;
        if (session != null)
        {
            sessionUser = (String) session.getAttribute(USER_KEY);
        }

        // If there is no user for the session, get the
        // user and password from the authentication header
        // in the request and validate
        String user = null;
        if (sessionUser == null)
        {
            user = validUser(req);
        }

        // If there is no user for the session and the user was
        // not authenticated from the request, force a login
        if ((sessionUser == null) && (user == null))
        {

            // The user is unauthorized to access this page.
            // Setting this status code will cause the browser
            // to prompt for a login
            resp.setStatus(HttpServletResponse.SC_UNAUTHORIZED);

            // Set the authentication realm
            resp.setHeader("WWW-Authenticate",
                           "BASIC realm=\"custom\"");

            // The following page will be displayed if the
            // user presses 'cancel'
            out.println("<html>");
            out.println("<head>");
            out.println("<title>Invalid User</title>");
            out.println("</head>");
            out.println("<body>");
            out.println("You are not currently logged in");
        }
        else
        {

            // If there is no user for the session, bind it now
            if ((sessionUser == null) && (session != null))
            {
                session.setAttribute(USER_KEY, user);
                sessionUser = user;
            }

            // Show the welcome page
            out.println("<html>");
            out.println("<head>");
            out.println("<title>Welcome</title>");
            out.println("</head>");
            out.println("<body>");
            out.println("<center><h2>Welcome " + sessionUser +
                    "!</h2></center>");
        }

        // Wrap up
        out.println("</body>");
        out.println("</html>");
        out.flush();
    }

    /**
     * Validate the user and password given in the authorization
     * header. This information is base 64 encoded and will
     * look something like:
     *
     *    Basic a2FybG1vc3M6YTFiMmMz
     *
     * @param req The request
     * @return The user name if valid or null if invalid
     */
    protected String validUser(HttpServletRequest req)
    {
        // Get the authorization header
        String encodedAuth = req.getHeader("Authorization");

        if (encodedAuth == null)
        {
            return null;
        }

        // The only authentication type we understand is BASIC
        if (!encodedAuth.toUpperCase().startsWith("BASIC"))
        {
            return null;
        }

        // Decode the rest of the string which will be the
        // username and password
        String decoded = Decoder.base64(encodedAuth.substring(6));

        // We should now have a string with the username and
        // password separated by a colon, such as:
        //     karlmoss:a1b2c3
        int idx = decoded.indexOf(":");
        if (idx < 0)
        {
            return null;
        }
        String user = decoded.substring(0, idx);
        String password = decoded.substring(idx + 1);

        // Validate the username and password.
        if (!validateUser(user, password))
        {
            user = null;
        }

        return user;
    }

    /**
     * Validates the username and password
     * @param user The user name
     * @param password The password
     * @return true if the username and password are valid
     */
    protected boolean validateUser(String user, String password)
    {
        boolean valid = false;

        if ((user != null) && (password != null))
        {

            // Just do a simple check now. A "real" check would
            // most likely use a database or LDAP server
            if (user.equals("karlmoss"))
            {
                valid = true;
            }
        }

        return valid;
    }

}

