/*
 * CounterApp
 *
 * Copyright (c) 1998-2002 Karl Moss. All Rights Reserved.
 *
 * Developed in conjunction with the book "Java Servlets Developer's
 * Guide" published by McGraw-Hill/Osborne Media Group
 *
 * You may study, use, modify, and distribute this software for any
 * purpose.
 *
 * This software is provided WITHOUT WARRANTY either expressed or
 * implied.
 *
 * @author  Karl Moss
 * @date Oct 18, 2001
 */

package com.omh.session;

import java.io.*;

public class CounterApp
{
    // The servlet url
    String servletURL;

    // The value of the session cookie
    String sessionCookie = null;

    /**
     * <p>Application entry point. This application requires
     * one parameter, which is the servlet URL
     */
    public static void main(String args[])
    {
        // Make sure we have an argument for the servlet URL
        if (args.length == 0)
        {
            System.out.println("\nServlet URL must be specified");
            return;
        }

        try {

            // Create a new object
            CounterApp app = new CounterApp(args[0]);

            // Get the count multiple times
            for (int i = 1; i <=5; i++)
            {
                int count = app.getCount();
                System.out.println("Pass " + i + " count=" + count);
            }
        }
        catch (Exception ex)
        {
            ex.printStackTrace();
        }

    }

    /**
     * Construct a new CounterApp object
     * @param url The servlet url
     */
    public CounterApp(String url)
    {
        servletURL = url;
    }

    /**
     * Invokes a counter servlet and returns the hit count
     * that was returned by the servlet
     */
    public int getCount() throws Exception
    {
        // Get the server URL
        java.net.URL url = new java.net.URL(servletURL);

        // Attempt to connect to the host
        java.net.URLConnection con = url.openConnection();

        // Set the session ID if necessary
        if (sessionCookie != null)
        {
            con.setRequestProperty("cookie", sessionCookie);
        }

        // Initialize the connection
        con.setUseCaches(false);
        con.setDoOutput(true);
        con.setDoInput(true);

        // Data will always be written to a byte array buffer so
        // that we can tell the server the length of the data
        ByteArrayOutputStream byteOut =
                new ByteArrayOutputStream();

        // Create the output stream to be used to write the
        // data to our buffer
        DataOutputStream out = new DataOutputStream(byteOut);

        // Send any data to the servlet here

        // Flush the data to the buffer
        out.flush();

        // Get our buffer to be sent
        byte buf[] = byteOut.toByteArray();

        // Set the content that we are sending
        con.setRequestProperty("Content-type",
            "application/octet-stream");

        // Set the length of the data buffer we are sending
        con.setRequestProperty("Content-length",
            "" + buf.length);

        // Get the output stream to the server and send our
        // data buffer
        DataOutputStream dataOut =
            new DataOutputStream(con.getOutputStream());
        dataOut.write(buf);

        // Flush the output stream and close it
        dataOut.flush();
        dataOut.close();

        // Get the input stream we can use to read the response
        DataInputStream in =
            new DataInputStream(con.getInputStream());

        // Read the data from the server
        int count = in.readInt();

        // Close the input stream
        in.close();

        // Get the session cookie if we haven't already
        if (sessionCookie == null)
        {
            String cookie = con.getHeaderField("set-cookie");
            if (cookie != null)
            {
                sessionCookie = parseCookie(cookie);
                System.out.println("Setting session ID=" +
                        sessionCookie);
            }
        }

        return count;
    }

    /**
     * Parses the given cookie and returns the cookie key
     * and value. For simplicity the key/ value is assumed
     * to be before the first ';', as in:
     *
     * JSESSIONID=3509823408122; path=/
     *
     * @param rawCookie The raw cookie data
     * @return The key/value of the cookie
     */
    public String parseCookie(String raw)
    {
        String c = raw;

        if (raw != null)
        {
            // Find the first ';'
            int endIndex = raw.indexOf(";");

            // Found a ';', assume the key/value is prior
            if (endIndex >= 0) {
                c = raw.substring(0, endIndex);
            }
        }

        return c;
    }
}
