/*
 * StockObject
 *
 * Copyright (c) 1998-2002 Karl Moss. All Rights Reserved.
 * 
 * Developed in conjunction with the book "Java Servlets Developer's
 * Guide" published by McGraw-Hill/Osborne Media Group
 *
 * You may study, use, modify, and distribute this software for any
 * purpose.
 *
 * This software is provided WITHOUT WARRANTY either expressed or
 * implied.
 *
 * @author  Karl Moss
 * @date Oct 25, 2001
 */

package com.omh.stocks;

/**
 * Represents a single stock entry
 */
public class StockObject
{
    public static final String NOT_AVAILABLE = "n/a";

    protected String symbol;
    protected String companyName;
    protected String lastTradeDate;
    protected String lastTradeTime;
    protected String price;
    protected String change;
    protected String openPrice;
    protected String highPrice;
    protected String lowPrice;
    protected String volume;
    protected long creationTime;

    /**
     * Construct a new StockObject
     * @param symbol The stock symbol
     */
    public StockObject(String symbol)
    {
        this.symbol = symbol;

        // Save the creation time
        creationTime = System.currentTimeMillis();

        // Setup default values
        companyName = NOT_AVAILABLE;
        lastTradeDate = NOT_AVAILABLE;
        lastTradeTime = NOT_AVAILABLE;
        price = NOT_AVAILABLE;
        change = NOT_AVAILABLE;
        openPrice = NOT_AVAILABLE;
        highPrice = NOT_AVAILABLE;
        lowPrice = NOT_AVAILABLE;
        volume = NOT_AVAILABLE;
    }

    /**
     * Sets the stock symbol
     * @param symbol The stock symbol
     */
    public void setSymbol(String symbol)
    {
        this.symbol = symbol;
    }

    /**
     * Gets the stock symbol
     * @return The stock symbol
     */
    public String getSymbol()
    {
        return symbol;
    }

    /**
     * Sets the company name
     * @param name The company name
     */
    public void setCompanyName(String name)
    {
        this.companyName = name;
    }

    /**
     * Gets the company name
     * @return The company name
     */
    public String getCompanyName()
    {
        return companyName;
    }

    /**
     * Sets the last trade date for the stock
     * @param lastTradeDate The last trade date
     */
    public void setLastTradeDate(String lastTradeDate)
    {
        this.lastTradeDate = lastTradeDate;
    }

    /**
     * Gets the last trade date for the stock
     * @return The last trade date
     */
    public String getLastTradeDate()
    {
        return lastTradeDate;
    }

    /**
     * Sets the last trade time for the stock
     * @param lastTradeTime The last trade time
     */
    public void setLastTradeTime(String lastTradeTime)
    {
        this.lastTradeTime = lastTradeTime;
    }

    /**
     * Gets the last trade time for the stock
     * @return The last trade time
     */
    public String getLastTradeTime()
    {
        return lastTradeTime;
    }

    /**
     * Sets the current price of the stock
     * @param price The current price
     */
    public void setPrice(String price)
    {
        this.price = price;
    }

    /**
     * Gets the current price of the stock
     * @return The current price
     */
    public String getPrice()
    {
        return price;
    }

    /**
     * Sets the change (delta) between the stock opening price
     * and the current price
     * @param change The price change
     */
    public void setChange(String change)
    {
        this.change = change;
    }

    /**
     * Gets the change (delta) between the stock opening price
     * and the current price
     * @return The price change
     */
    public String getChange()
    {
        return change;
    }

    /**
     * Sets the opening price of the stock
     * @param openPrice The opening price
     */
    public void setOpenPrice(String openPrice)
    {
        this.openPrice = openPrice;
    }

    /**
     * Gets the opening price of the stock
     * @return The opening price
     */
    public String getOpenPrice()
    {
        return openPrice;
    }

    /**
     * Sets the high price of the stock for the day
     * @param highPrice The high price
     */
    public void setHighPrice(String highPrice)
    {
        this.highPrice = highPrice;
    }

    /**
     * Gets the high price of the stock for the day
     * @return The high price
     */
    public String getHighPrice()
    {
        return highPrice;
    }

    /**
     * Sets the low price of the stock for the day
     * @param lowPrice The low price
     */
    public void setLowPrice(String lowPrice)
    {
        this.lowPrice = lowPrice;
    }

    /**
     * Gets the low price of the stock for the day
     * @return The low price
     */
    public String getLowPrice()
    {
        return lowPrice;
    }

    /**
     * Sets the volume (shares traded) of the stock for the day
     * @param volume The stock volume
     */
    public void setVolume(String volume)
    {
        this.volume = volume;
    }

    /**
     * Gets the volume (shares traded) of the stock for the day
     * @return The stock volume
     */
    public String getVolume()
    {
        return volume;
    }

    /**
     * Gets the creation time of this object
     * @return The creation time
     */
    public long getCreationTime()
    {
        return creationTime;
    }

    public String toString()
    {
        StringBuffer sb = new StringBuffer(getSymbol());
        sb.append(" NAME=");
        sb.append(getCompanyName());
        sb.append(" LAST=");
        sb.append(getLastTradeDate());
        sb.append(" ");
        sb.append(getLastTradeTime());
        sb.append(" PRICE=");
        sb.append(getPrice());
        sb.append(" CHG=");
        sb.append(getChange());
        sb.append(" OPEN=");
        sb.append(getOpenPrice());
        sb.append(" HIGH=");
        sb.append(getHighPrice());
        sb.append(" LOW=");
        sb.append(getLowPrice());
        sb.append(" VOL=");
        sb.append(getVolume());
        return sb.toString();
    }
}
