/*
 * StockReader
 *
 * Copyright (c) 1998-2002 Karl Moss. All Rights Reserved.
 * 
 * Developed in conjunction with the book "Java Servlets Developer's
 * Guide" published by McGraw-Hill/Osborne Media Group
 *
 * You may study, use, modify, and distribute this software for any
 * purpose.
 *
 * This software is provided WITHOUT WARRANTY either expressed or
 * implied.
 *
 * @author  Karl Moss
 * @date Oct 25, 2001
 */

package com.omh.stocks;

import java.util.*;
import java.io.*;
import java.net.*;

public class StockReader
{
    // Hashtable used for stock quote cache. The key is the stock
    // symbol and the element is a StockObject
    static Hashtable quoteCache = new Hashtable();

    // Life of a cached stock quote (in milliseconds)
    static long expiration = 60000;

    /**
     * Gets a delayed stock quote for each of the given stock symbols. Returns
     * an array of StockObject objects.
     * @param symbols An array of stock symbols to get info for
     * @return A StockObject array with the info for each stock symbol
     */
    public static StockObject[] getStockInfo(String symbols[])
        throws Exception
    {
        if (symbols == null)
        {
            return null;
        }

        // Create our new array
        StockObject[] stocks = new StockObject[symbols.length];

        long now = System.currentTimeMillis();

        // The number of stocks that need to be retrieved
        int count = 0;

        // The query string to pass to Yahoo
        String query = "http://quote.yahoo.com/d/quotes.csv?s=";

        // Loop through the symbols checking our cache. If we have a cached
        // symbol that is not expired, use it. Otherwise get the stock
        // info from Yahoo
        for (int i = 0; i < symbols.length; i++)
        {
            String symbol = symbols[i].toUpperCase();

            // Check the cache
            StockObject cachedQuote = (StockObject) quoteCache.get(symbol);
            if (cachedQuote != null)
            {
                if ((cachedQuote.getCreationTime() + expiration) < now)
                {
                    // Expired!
                    cachedQuote = null;
                    // We could remove it from the cache but we'll just be
                    // adding it back in, so we'll leave it
                }
            }

            if (cachedQuote != null)
            {
                stocks[i] = cachedQuote;
            }
            else
            {
                if (count > 0)
                {
                    query += "%20";
                }
                query += symbol;
                count++;
            }
        }

        // If there are any stocks to be looked up, do it now. We'll know this
        // if the count > 0
        if (count > 0)
        {
            BufferedReader reader = null;
            try
            {
                query += "&f=sl1d1t1c1ohgvn&e=.csv";
                reader = getInfoFromUrl(query);

                // Now go back through our list of symbols and get the info out
                for (int i = 0; i < symbols.length; i++)
                {
                    // Skip the symbols we got out of the cache
                    if (stocks[i] != null)
                    {
                        continue;
                    }

                    String line = reader.readLine();
                    if (line != null)
                    {
                        StockObject o = parseStockObject(line);

                        // Place the info in our cache
                        quoteCache.put(o.getSymbol(), o);

                        // Set in the return array
                        stocks[i] = o;
                    }
                }
            }
            finally
            {
                if (reader != null)
                {
                    try {
                        reader.close();
                    }
                    catch (Exception ex) {
                        // Ignore errors on close
                    }
                }
            }
        }

        return stocks;
    }

    /**
     * Gets stock info from a url
     * @param url The url to retrieve
     * @return A BufferedReader used to read the response. The caller is responsible
     * for closing the reader. Returns null if a connection cannot be made
     */
    protected static BufferedReader getInfoFromUrl(String url)
        throws Exception
    {
        BufferedReader reader = null;
        URL u = new URL(url);
        URLConnection connection = u.openConnection();
        connection.setUseCaches(false);
        connection.setDoInput(true);
        reader = new BufferedReader(
                new InputStreamReader(connection.getInputStream()));
        return reader;
    }

    /**
     * Parses the line read from the url and populates a StockObject
     */
    protected static StockObject parseStockObject(String line)
    {
        int pos = 0;
        StockObject o = null;

        // Use a StringTokenizer because it's easy. Beware that
        // StringTokenizer is not very efficient
        StringTokenizer st = new StringTokenizer(line, ",");
        while (st.hasMoreElements())
        {
            String data = (String) st.nextElement();

            // Strip quotes
            if (data.startsWith("\"") &&
                data.endsWith("\""))
            {
                data = data.substring(1, data.length() - 1);
            }

            // Set attributes on the StockObject
            switch(pos)
            {
            case 0:
                // SYMBOL
                o = new StockObject(data);
                break;
            case 1:
                // PRICE
                o.setPrice(data);
                break;
            case 2:
                // DATE
                o.setLastTradeDate(data);
                break;
            case 3:
                // TIME
                o.setLastTradeTime(data);
                break;
            case 4:
                // CHANGE
                o.setChange(data);
                break;
            case 5:
                // OPEN
                o.setOpenPrice(data);
                break;
            case 6:
                // HIGH
                o.setHighPrice(data);
                break;
            case 7:
                // LOW
                o.setLowPrice(data);
                break;
            case 8:
                // VOLUME
                o.setVolume(data);
                break;
            case 9:
                // NAME
                o.setCompanyName(data);
                break;
            }
            pos++;
        }
        return o;
    }

    /**
     * Stand-alone test
     */
    public static void main(String args[])
    {
        if (args.length == 0)
        {
            System.out.println("Stock symbol(s) must be given.");
        }
        else
        {
            try
            {
                StockObject[] so = getStockInfo(args);
                for (int i = 0; i < so.length; i++)
                {
                    System.out.println(so[i]);
                }
            }
            catch (Exception ex)
            {
                ex.printStackTrace();
            }
        }
    }
}
