/*
 * BaseTunnelClient
 *
 * Copyright (c) 1998-2002 Karl Moss. All Rights Reserved.
 *
 * Developed in conjunction with the book "Java Servlets Developer's
 * Guide" published by McGraw-Hill/Osborne Media Group
 *
 * You may study, use, modify, and distribute this software for any
 * purpose.
 *
 * This software is provided WITHOUT WARRANTY either expressed or
 * implied.
 *
 * @author  Karl Moss
 * @date Nov 24, 2001
 */

package com.omh.tunnel.client;

import java.io.*;
import java.net.URL;
import java.net.URLConnection;

/**
 * This class provides base functionality common to all
 * types of tunnel clients
 */

public abstract class BaseTunnelClient
{
    // The connection url
    URL _url;

    // The server object handle
    private int objectHandle;

    // The session cookie
    private String sessionCookie;

    /**
     * Sets the URL of the connection
     *
     * @param url Connection URL
     */
    public void _setURL(URL url)
    {
        _url = url;
    }

    /**
     * Gets the URL of the connection
     *
     * @return The connection URL
     */
    public URL _getURL()
    {
        return _url;
    }

    /**
     * Gets an input stream to be used for reading data
     * from the connection. Must be implemented by the client.
     *
     * @param in Input stream from the connection URL
     * @return Input stream to read data from the connection
     */
    public abstract DataInput _getInputStream(InputStream in)
        throws IOException;

    /**
     * Gets an output stream to be used for writing data to
     * an internal buffer. The buffer will be written to the
     * connection. Must be implemented by the client.
     *
     * @param buffer Buffer to hold the output data
     * @return Output stream to write data to the buffer
     */
    public abstract DataOutput _getOutputStream(
            ByteArrayOutputStream buffer)
        throws IOException;

    /**
     * Flushes the any buffered data to the output stream. The
     * default implementation does nothing
     *
     * @param out Output stream to flush
     */
    public void _flush(DataOutput out) throws IOException
    {
    }

    /**
     * Closes the input stream. The default implementation does
     * nothing
     *
     * @param in Input stream to close
     */
    public void _close(DataInput in) throws IOException
    {
    }

    /**
     * Initializes the client. Also makes a server request
     * to initialize the server as well.
     */
    public void _initialize() throws TunnelException
    {
        try
        {
            // Create a new buffer that will hold our data
            ByteArrayOutputStream buffer = new ByteArrayOutputStream();

            // Create a method header. An ordinal value of -1 is
            // reserved for initializing the server
            _createHeader(buffer, -1);

            // Invoke the method. This will send the initialization
            // header to the server
            DataInput in = _invokeMethod(buffer.toByteArray());

            // Get the handle to the object that was just
            // created
            objectHandle = in.readInt();

            // Close properly
            _close(in);
        }
        catch (IOException ex)
       {
            // Re-throw as a tunnel exception
            ex.printStackTrace();
            throw new TunnelException(ex.getMessage());
        }
    }

    /**
     * Starts a method by creating the method header.
     * The header consists of the method ordinal to invoke.
     *
     * @param buffer Buffer to hold the header data
     * @param ordinal Method ordinal to invoke on the server
     * @return Output stream to be used to send parameters
     */
    public DataOutput _createHeader(ByteArrayOutputStream buffer,
             int ordinal)
        throws TunnelException
    {
        try
        {
            // Get an output stream use to write data to the buffer
            DataOutput out = _getOutputStream(buffer);

            // Write the method ordinal
            out.writeInt(ordinal);

            // If we are not initializing the object we need to send
            // the object handle along with the header
            if (ordinal != -1)
            {
                out.writeInt(objectHandle);
            }

            _flush(out);
            return out;
        }
        catch (IOException ex)
        {
            // Re-throw as a tunnel exception
            ex.printStackTrace();
            throw new TunnelException(ex.getMessage());
        }
    }

    /**
     * Sends the given buffer that will cause a remote
     * method to be invoked.
     *
     * @param buffer Buffer containing data to send to the server
     * @return Input stream to be used to read the response from
     * the server
     */
    public DataInput _invokeMethod(byte buf[])
        throws TunnelException
    {
        DataInput in = null;

        try
        {
            // Get the server URL
            URL url = _getURL();
            if (url == null)
            {
                throw new IOException("Server URL has not been set");
            }

            // Attempt to connect to the host
            URLConnection con = url.openConnection();

            // Initialize the connection
            con.setUseCaches(false);
            con.setDoOutput(true);
            con.setDoInput(true);

            // Set the content that we are sending
            con.setRequestProperty("Content-type",
                    "application/octet-stream");

            // Set the length of the data buffer we are sending
            con.setRequestProperty("Content-length",
                    "" + buf.length);

            // Set the session cookie if we have one
            if (sessionCookie != null)
            {
                con.setRequestProperty("cookie", sessionCookie);
            }

            // Get the output stream to the server and send our
            // data buffer
            DataOutputStream out =
                    new DataOutputStream(con.getOutputStream());
            out.write(buf);

            // Flush the output stream and close it
            out.flush();
            out.close();

            // Get the input stream we can use to read the response
            in = _getInputStream(con.getInputStream());

            // The server will always respond with an int value
            // that will either be the method ordinal that was
            // invoked, or a -2 indicating an exception was thrown
            // from the server
            int ordinal = in.readInt();

            // Check for an exception on the server.
            if (ordinal == -2)
            {
                // Read the exception message and throw it
                String msg = in.readUTF();
                throw new TunnelException(msg);
            }

            // Get the session cookie if we don't have one yet
            if (sessionCookie == null)
            {
                String cookie = con.getHeaderField("set-cookie");
                if (cookie != null)
                {
                    sessionCookie = parseCookie(cookie);
                }
            }
        }
        catch (IOException ex)
        {
            // Re-throw as a tunnel exception
            ex.printStackTrace();
            throw new TunnelException(ex.getMessage());
        }

        // Return the input stream to be used to read the rest
        // of the response from the server
        return in;
    }

    /**
     * Gets the cookie name and value from a cookie field
     */
    public static String parseCookie(String raw)
    {
        String c = raw;
        if (raw != null)
        {
            // Find the first ';'
            int endPos = raw.indexOf(';');

            if (endPos >= 0)
            {
                c = raw.substring(0, endPos);
            }
        }
        return c;
    }
}
